/*
 * Drizzle PHP Extension
 *
 * All rights reserved.
 * Copyright (C) 2008 Eric Day (eday@oddments.org)
 *             James M. Luedke (contact@jamesluedke.com)
 *
 *
 * Use and distribution licensed under the PHP license.  See
 * the LICENSE file in this directory for full text.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_drizzle.h"

#include "zend_exceptions.h"
#include "zend_interfaces.h"

#include <libdrizzle/drizzle_client.h>

/* XXX Compatibility Macros
 * If there is a better way to do this someone please let me know.
 * Also which is the prefered method now? ZVAL_ADDREF or Z_ADDREF_P ?
 * -jluedke */
#if PHP_VERSION_ID < 50300
# ifndef Z_ADDREF_P
#  define Z_ADDREF_P(pz)				(pz)->refcount++
#  define Z_ADDREF_PP(ppz)				Z_ADDREF_P(*(ppz))
#  define Z_ADDREF(z)					Z_ADDREF_P(&(z))
# endif

# ifndef Z_DELREF_P
#  define Z_DELREF_P(pz)				(pz)->refcount--
#  define Z_DELREF_PP(ppz)				Z_DELREF_P(*(ppz))
#  define Z_DELREF(z)					Z_DELREF_P(&(z))
# endif
#endif



/* XXX I hate to do this but they changed PHP_ME_MAPPING between versions.
 * in order to make the module compile on versions < 5.2 this is required */
#if (PHP_MAJOR_VERSION == 5) && (PHP_MINOR_VERSION < 2)
#	define __PHP_ME_MAPPING(__name, __func, __arg, __flags) PHP_ME_MAPPING(__name, __func, __arg)
#else
#	define __PHP_ME_MAPPING(__name, __func, __arg, __flags) PHP_ME_MAPPING(__name, __func, __arg, __flags)
#endif

/* 
 * {{{ START arginfo 
 */
ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_version, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_version, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_create, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_clone, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_clone, 0, 0, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_free, 0, 0, 0)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_error, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_error, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_errno, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_errno, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_error_code, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_error_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_sqlstate, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_sqlstate, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_options, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_options, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_set_options, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_set_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_add_options, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_add_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_remove_options, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_remove_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_return_code, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_return_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_create, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_create, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_clone, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_clone, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_free, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_free, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_fd, 0, 0, 0)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_fd, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_fd, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, fd)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_fd, 0, 0, 0)
    ZEND_ARG_INFO(0, fd)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_close, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_close, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_wait, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_wait, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_ready, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_ready, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_drizzle, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_drizzle, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_error, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_error, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_errno, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_errno, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_error_code, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_error_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_sqlstate, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_sqlstate, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_options, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_options, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_options, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_add_options, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_add_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_remove_options, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_remove_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_host, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_host, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_port, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_port, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_tcp, 0, 0, 3)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, host)
    ZEND_ARG_INFO(0, port)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_tcp, 0, 0, 2)
    ZEND_ARG_INFO(0, host)
    ZEND_ARG_INFO(0, port)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_uds, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_uds, 0, 0, 2)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_uds, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, uds)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_uds, 0, 0, 1)
    ZEND_ARG_INFO(0, uds)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_user, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_user, 0, 0, 0)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_password, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_password, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_auth, 0, 0, 3)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_auth, 0, 0, 2)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_db, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_db, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_db, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, db)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_db, 0, 0, 1)
    ZEND_ARG_INFO(0, db)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_context, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_context, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_set_context, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_set_context, 0, 0, 1)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_protocol_version, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_protocol_version, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_server_version, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_server_version, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_server_version_number, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_server_version_number, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_thread_id, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_thread_id, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_scramble, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_scramble, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_capabilities, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_capabilities, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_charset, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_charset, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_status, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_status, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_max_packet_size, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_max_packet_size, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_return_code, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_return_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_add_tcp, 0, 0, 7)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, host)
    ZEND_ARG_INFO(0, port)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
    ZEND_ARG_INFO(0, db)
    ZEND_ARG_INFO(0, option)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_add_tcp, 0, 0, 6)
    ZEND_ARG_INFO(0, host)
    ZEND_ARG_INFO(0, port)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
    ZEND_ARG_INFO(0, db)
    ZEND_ARG_INFO(0, option)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_add_uds, 0, 0, 6)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, uds)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
    ZEND_ARG_INFO(0, db)
    ZEND_ARG_INFO(0, option)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_add_uds, 0, 0, 5)
    ZEND_ARG_INFO(0, uds)
    ZEND_ARG_INFO(0, user)
    ZEND_ARG_INFO(0, password)
    ZEND_ARG_INFO(0, db)
    ZEND_ARG_INFO(0, option)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_connect, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_connect, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_handshake_server_read, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_handshake_server_read, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_handshake_client_write, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_handshake_client_write, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_quit, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_quit, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_select_db, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, db)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_select_db, 0, 0, 1)
    ZEND_ARG_INFO(0, db)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_shutdown, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, level)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_shutdown, 0, 0, 1)
    ZEND_ARG_INFO(0, level)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_ping, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_ping, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_con_command_write, 0, 0, 4)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, command)
    ZEND_ARG_INFO(0, data)
    ZEND_ARG_INFO(0, total)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_con_command_write, 0, 0, 3)
    ZEND_ARG_INFO(0, command)
    ZEND_ARG_INFO(0, data)
    ZEND_ARG_INFO(0, total)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query, 0, 0, 2)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, query)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query, 0, 0, 1)
    ZEND_ARG_INFO(0, query)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_inc, 0, 0, 3)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, query)
    ZEND_ARG_INFO(0, total)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_inc, 0, 0, 2)
    ZEND_ARG_INFO(0, query)
    ZEND_ARG_INFO(0, total)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_add, 0, 0, 5)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, query)
    ZEND_ARG_INFO(0, options)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_add, 0, 0, 4)
    ZEND_ARG_INFO(0, con_object)
    ZEND_ARG_INFO(0, query)
    ZEND_ARG_INFO(0, options)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_create, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_create, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_free, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_free, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_con, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_con, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_set_con, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_set_con, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_result, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_result, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_string, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_string, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_set_string, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, query)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_set_string, 0, 0, 1)
    ZEND_ARG_INFO(0, query)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_options, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_options, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_set_options, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_set_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_add_options, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_add_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_remove_options, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_remove_options, 0, 0, 1)
    ZEND_ARG_INFO(0, options)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_context, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_context, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_set_context, 0, 0, 2)
    ZEND_ARG_INFO(0, query_object)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_set_context, 0, 0, 1)
    ZEND_ARG_INFO(0, data)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_run, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_run, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_run_all, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_run_all, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_query_return_code, 0, 0, 1)
    ZEND_ARG_INFO(0, query_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_query_return_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_escape_string, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, string)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_escape_string, 0, 0, 1)
    ZEND_ARG_INFO(0, string)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_hex_string, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, string)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_hex_string, 0, 0, 1)
    ZEND_ARG_INFO(0, string)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_create, 0, 0, 1)
    ZEND_ARG_INFO(0, drizzle_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_create, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_clone, 0, 0, 2)
    ZEND_ARG_INFO(0, drizzle_object)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_clone, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_free, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_free, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_drizzle_con, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_drizzle_con, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_eof, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_eof, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_info, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_info, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_error, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_error, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_error_code, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_error_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_sqlstate, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_sqlstate, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_warning_count, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_warning_count, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_insert_id, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_insert_id, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_affected_rows, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_affected_rows, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_column_count, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_column_count, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_row_count, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_row_count, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_read, 0, 0, 1)
    ZEND_ARG_INFO(0, con_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_read, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_buffer, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_buffer, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_row_size, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_row_size, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_result_return_code, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_result_return_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_create, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_create, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_free, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_drizzle_result, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_drizzle_result, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_catalog, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_catalog, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_db, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_db, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_table, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_table, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_orig_table, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_orig_table, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_name, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_name, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_orig_name, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_orig_name, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_charset, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_charset, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_size, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_size, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_max_size, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_max_size, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_type, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_type, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_type_drizzle, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_type_drizzle, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_flags, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_flags, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_decimals, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_decimals, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_default_value, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_default_value, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_skip, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_skip, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_read, 0, 0, 0)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_read, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_buffer, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_buffer, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_next, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_next, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_prev, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_prev, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_seek, 0, 0, 0)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_seek, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_index, 0, 0, 2)
    ZEND_ARG_INFO(0, result_object)
    ZEND_ARG_INFO(0, column_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_index, 0, 0, 1)
    ZEND_ARG_INFO(0, column_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_current, 0, 0, 1)
    ZEND_ARG_INFO(0, column_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_current, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_column_return_code, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_column_return_code, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_read, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_read, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_buffer, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_buffer, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_field_sizes, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_field_sizes, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_next, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_next, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_prev, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_prev, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_seek, 0, 0, 2)
    ZEND_ARG_INFO(0, result_object)
    ZEND_ARG_INFO(0, row_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_seek, 0, 0, 1)
    ZEND_ARG_INFO(0, row_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_index, 0, 0, 2)
    ZEND_ARG_INFO(0, result_object)
    ZEND_ARG_INFO(0, row_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_index, 0, 0, 0)
    ZEND_ARG_INFO(0, row_index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_row_current, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_row_current, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_field_read, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_field_read, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_field_buffer, 0, 0, 1)
    ZEND_ARG_INFO(0, result_object)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_oo_drizzle_field_buffer, 0, 0, 0)
ZEND_END_ARG_INFO()

#if jluedke_0
ZEND_BEGIN_ARG_INFO_EX(arginfo_drizzle_field_free, 0, 0, 0)
ZEND_END_ARG_INFO()
#endif

/* 
 * }}} end arginfo
 */

/*
 * Object types and structures.
 */

typedef enum {
	DRIZZLE_OBJ_CREATED= (1 << 0)
} drizzle_obj_flags_t;

typedef struct {
	zend_object std;
	drizzle_obj_flags_t flags;
	drizzle_st drizzle;
    drizzle_return_t ret;
} drizzle_obj;

typedef enum {
	DRIZZLE_CON_OBJ_CREATED= (1 << 0),
	DRIZZLE_CON_OBJ_DEAD=    (1 << 1)
} drizzle_con_obj_flags_t;

typedef struct {
	zend_object std;
	zend_object_value value;
	drizzle_con_obj_flags_t flags;
	drizzle_con_st *con;
	zval *drizzle;
	zval *data;
    drizzle_return_t ret;
} drizzle_con_obj;

typedef enum {
	DRIZZLE_QUERY_OBJ_CREATED= (1 << 0),
	DRIZZLE_QUERY_OBJ_DEAD=    (1 << 1)
} drizzle_query_obj_flags_t;

typedef struct {
	zend_object std;
	zend_object_value value;
	drizzle_query_obj_flags_t flags;
	drizzle_query_st *query;
	zval *drizzle;
	zval *zquery;
	zval *data;
    drizzle_return_t ret;
} drizzle_query_obj;

typedef enum {
	DRIZZLE_RESULT_OBJ_CREATED= (1 << 0)
} drizzle_result_obj_flags_t;

typedef struct {
	zend_object std;
	drizzle_result_obj_flags_t flags;
	drizzle_result_st *result;
	zval *drizzle;
	zval *field_sizes;
    drizzle_return_t ret;
} drizzle_result_obj;

typedef enum {
	DRIZZLE_COLUMN_OBJ_CREATED= (1 << 0)
} drizzle_column_obj_flags_t;

typedef struct {
	zend_object std;
	drizzle_column_obj_flags_t flags;
	drizzle_column_st *column;
	zval *result;
    drizzle_return_t ret;
} drizzle_column_obj;

/*
 * Object variables.
 */

zend_class_entry *drizzle_ce;
static zend_object_handlers drizzle_obj_handlers;
zend_class_entry *drizzle_con_ce;
static zend_object_handlers drizzle_con_obj_handlers;
zend_class_entry *drizzle_query_ce;
static zend_object_handlers drizzle_query_obj_handlers;
zend_class_entry *drizzle_result_ce;
static zend_object_handlers drizzle_result_obj_handlers;
zend_class_entry *drizzle_column_ce;
static zend_object_handlers drizzle_column_obj_handlers;
zend_class_entry *drizzle_exception_ce;
/* XXX
static zend_object_handlers drizzle_exception_obj_handlers;
*/

/*
 * Helper macros.
 */

#define DRIZZLE_ZPP(__return, __args, ...) { \
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, __args, \
		                        __VA_ARGS__) == FAILURE) { \
		__return; \
	} \
}

#define DRIZZLE_ZPMP(__return, __args, ...) { \
	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), \
		                               "O" __args, __VA_ARGS__) == FAILURE) { \
		__return; \
	} \
	obj= zend_object_store_get_object(zobj TSRMLS_CC); \
}

#define DRIZZLE_RESULT_INIT(__zval, __result, __drizzle_zval) { \
	Z_TYPE_P(__zval)= IS_OBJECT; \
	object_init_ex(__zval, drizzle_result_ce); \
	(__result)= zend_object_store_get_object(__zval TSRMLS_CC); \
	(__result)->drizzle= __drizzle_zval; \
	Z_ADDREF_P(__drizzle_zval); \
}

#define DRIZZLE_ZVAL_DONE(__zval) { \
	if ((__zval) != NULL) { \
		if (READY_TO_DESTROY(__zval)) { \
			zval_dtor(__zval); \
			FREE_ZVAL(__zval); \
		} \
		else {\
			Z_DELREF_P(__zval); \
		} \
	} \
}

#define DRIZZLE_RESULT_ERROR_CHECK(__ret, __result, __result_zval, __con) { \
	if ((__ret) != DRIZZLE_RETURN_OK && (__ret) != DRIZZLE_RETURN_IO_WAIT) { \
		if ((__ret) == DRIZZLE_RETURN_ERROR_CODE && (__result) != NULL) { \
			php_error_docref(NULL TSRMLS_CC, E_WARNING, \
			                 "%s", drizzle_result_error(__result)); \
			drizzle_result_free(__result); \
		} \
		else { \
			php_error_docref(NULL TSRMLS_CC, E_WARNING, \
			                 "%s", drizzle_con_error(__con)); \
		} \
		zval_dtor(__result_zval); \
		RETURN_FALSE; \
	} \
}

#define DRIZZLE_EXCEPTION(__error, __error_code) { \
	zend_throw_exception(drizzle_exception_ce, __error, __error_code TSRMLS_CC); \
    return; \
}

#define DRIZZLE_VALIDATE_OPTIONS(__options) (__options & ~DRIZZLE_ALLOCATED)
#define DRIZZLE_COLUMN_VALIDATE_OPTIONS(__options) (__options & ~(DRIZZLE_COLUMN_ALLOCATED))
#define DRIZZLE_CON_VALIDATE_OPTIONS(__options) (__options & ~(DRIZZLE_CON_ALLOCATED))
#define DRIZZLE_QUERY_VALIDATE_OPTIONS(__options) (__options & ~(DRIZZLE_QUERY_ALLOCATED))
#define DRIZZLE_RESULT_VALIDATE_OPTIONS(__options) (__options & ~(DRIZZLE_RESULT_ALLOCATED))


/*
 * Methods for drizzle.
 */

PHP_METHOD(Drizzle, __construct) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce);

	if (drizzle_create(&(obj->drizzle)) == NULL) {
		DRIZZLE_EXCEPTION("Memory allocation failure.", 0);
	}

	obj->flags|= DRIZZLE_OBJ_CREATED;
}


static void drizzle_obj_free(void *object TSRMLS_DC) {
	drizzle_obj *obj= (drizzle_obj *)object;

	if (obj->flags & DRIZZLE_OBJ_CREATED) {
		drizzle_free(&(obj->drizzle));
	}

	zend_object_std_dtor(&(obj->std) TSRMLS_CC);
	efree(object);
}

static inline zend_object_value
drizzle_obj_new_ex(zend_class_entry *class_type,
                   drizzle_obj **drizzle_ptr TSRMLS_DC) {
	drizzle_obj *obj;
	zend_object_value value;
	zval *tmp;

	obj= emalloc(sizeof(drizzle_obj));
	memset(obj, 0, sizeof(drizzle_obj));

	if (drizzle_ptr) {
		*drizzle_ptr= obj;
	}

	zend_object_std_init(&(obj->std), class_type TSRMLS_CC);
#if PHP_VERSION_ID < 50399
	zend_hash_copy(obj->std.properties, &(class_type->default_properties),
	              (copy_ctor_func_t)zval_add_ref, (void *)(&tmp),
	               sizeof(zval *));
#else
	object_properties_init(&(obj->std), class_type);
#endif

	value.handle= zend_objects_store_put(obj,
	                   (zend_objects_store_dtor_t)zend_objects_destroy_object,
	                   (zend_objects_free_object_storage_t)drizzle_obj_free,
	                    NULL TSRMLS_CC);
	value.handlers= &drizzle_obj_handlers;

	return value;
}

static zend_object_value
drizzle_obj_new(zend_class_entry *class_type TSRMLS_DC) {
	return drizzle_obj_new_ex(class_type, NULL TSRMLS_CC);
}

/*
 * Functions from drizzle.h
 */

/* {{{ proto string drizzle_version()
   Get library version string */
PHP_FUNCTION(drizzle_version) {
	RETURN_STRING((char *)drizzle_version(), 1)
}
/* }}} */

/* {{{ proto object drizzle_create()
   Initialize a library instance structure. */
PHP_FUNCTION(drizzle_create) {
	drizzle_obj *obj;

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_ce);
	obj= zend_object_store_get_object(return_value TSRMLS_CC);

	if (drizzle_create(&(obj->drizzle)) == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL()
	}

	obj->flags|= DRIZZLE_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_clone()
   Clone a library instance structure. */
PHP_FUNCTION(drizzle_clone) {
	zval *zobj;
	drizzle_obj *obj;
	drizzle_obj *new;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_ce);
	new= zend_object_store_get_object(return_value TSRMLS_CC);

	if (drizzle_clone(&(new->drizzle), &(obj->drizzle)) == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL()
	}

	new->flags|= DRIZZLE_OBJ_CREATED;
}
/* }}} */

/* {{{ proto void drizzle_free(object drizzle)
   Free a library instance structure. */
PHP_FUNCTION(drizzle_free) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPP(RETURN_NULL(), "O", &zobj, drizzle_ce)
	obj= zend_object_store_get_object(zobj TSRMLS_CC);

	if (obj->flags & DRIZZLE_OBJ_CREATED) {
		drizzle_free(&(obj->drizzle));
		obj->flags&= ~DRIZZLE_OBJ_CREATED;
	}
}
/* }}} */

/* {{{ proto string drizzle_error(object drizzle)
   Return an error string for last library error encountered. */
PHP_FUNCTION(drizzle_error) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_STRING((char *)drizzle_error(&(obj->drizzle)), 1)
}
/* }}} */

/* {{{ proto int drizzle_errno(object drizzle)
   Return an error string for last library error encountered. */
PHP_FUNCTION(drizzle_errno) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_LONG(drizzle_errno(&(obj->drizzle)))
}
/* }}} */

/* {{{ proto int drizzle_error_code(object drizzle)
   Get server defined error code for the last result read. */
PHP_FUNCTION(drizzle_error_code) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_LONG(drizzle_error_code(&(obj->drizzle)))
}
/* }}} */

/* {{{ proto string drizzle_sqlstate(object drizzle)
   Get SQL state code for the last result read. */
PHP_FUNCTION(drizzle_sqlstate) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_STRING((char *)drizzle_sqlstate(&(obj->drizzle)), 1)
}
/* }}} */

/* {{{ proto int drizzle_options(object drizzle)
   Get options for a library instance structure. */
PHP_FUNCTION(drizzle_options) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_LONG(drizzle_options(&(obj->drizzle)))
}
/* }}} */

/* {{{ proto void drizzle_set_options(object drizzle, int options)
   Set options for a library instance structure. */
PHP_FUNCTION(drizzle_set_options) {
	zval *zobj;
	drizzle_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_ce, &options)

	drizzle_set_options(&(obj->drizzle), DRIZZLE_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_add_options(object drizzle, int options)
   Add options for a library instance structure. */
PHP_FUNCTION(drizzle_add_options) {
	zval *zobj;
	drizzle_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_ce, &options)

	drizzle_add_options(&(obj->drizzle), DRIZZLE_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_remove_options(object drizzle, int options)
   Remove options for a library instance structure. */
PHP_FUNCTION(drizzle_remove_options) {
	zval *zobj;
	drizzle_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_ce, &options)

	drizzle_remove_options(&(obj->drizzle), DRIZZLE_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_return_code(object drizzle)
   Get the last return result for object */
PHP_FUNCTION(drizzle_return_code) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	RETURN_LONG(obj->ret);
}
/* }}} */

/*
 * Methods for drizzle_con.
 */

void drizzle_con_obj_context_free(drizzle_con_st *con, void *data) {
	drizzle_con_obj *obj= (drizzle_con_obj *)data;
    TSRMLS_FETCH();

	DRIZZLE_ZVAL_DONE(obj->data)
	efree(obj);
}

PHP_METHOD(DrizzleCon, __construct) {
	zval *zobj;
	drizzle_con_obj *obj;
	zval *zdrizzle;
	drizzle_obj *drizzle;

	DRIZZLE_ZPMP(DRIZZLE_EXCEPTION("A drizzle object is required", 0), "O", 
							&zobj, drizzle_con_ce, &zdrizzle, drizzle_ce)

	drizzle= zend_object_store_get_object(zdrizzle TSRMLS_CC);

	obj->drizzle= zdrizzle;
	Z_ADDREF_P(zdrizzle);

	obj->con= drizzle_con_create(&(drizzle->drizzle), NULL);
	if (obj->con == NULL) {
		DRIZZLE_EXCEPTION("Memory Allocation Failure", 0);
	}

	obj->flags|= DRIZZLE_CON_OBJ_CREATED;
	drizzle_con_set_context(obj->con, obj);
	drizzle_con_set_context_free_fn(obj->con, drizzle_con_obj_context_free);
}

static void drizzle_con_obj_free(void *object TSRMLS_DC)
{
	drizzle_con_obj *obj= (drizzle_con_obj *)object;

	/* We don't call drizzle_con_free here since the drizzle object 
	 * can still use the connection internally. */
	DRIZZLE_ZVAL_DONE(obj->drizzle)

	zend_object_std_dtor(&(obj->std) TSRMLS_CC);

	if (obj->flags & DRIZZLE_CON_OBJ_CREATED) {
		obj->flags|= DRIZZLE_CON_OBJ_DEAD;
	} else {
		DRIZZLE_ZVAL_DONE(obj->data)
		efree(object);
	}
}

static inline zend_object_value
drizzle_con_obj_new_ex(zend_class_entry *class_type,
                       drizzle_con_obj **con_ptr TSRMLS_DC) {
	drizzle_con_obj *obj;
	zval *tmp;

	obj= emalloc(sizeof(drizzle_con_obj));
	memset(obj, 0, sizeof(drizzle_con_obj));

	if (con_ptr) {
		*con_ptr= obj;
	}

	zend_object_std_init(&(obj->std), class_type TSRMLS_CC);
#if PHP_VERSION_ID < 50399
	zend_hash_copy(obj->std.properties, &(class_type->default_properties),
	               (copy_ctor_func_t)zval_add_ref, (void *)(&tmp),
	               sizeof(zval *));
#else
	object_properties_init(&(obj->std), class_type);
#endif
	obj->value.handle= zend_objects_store_put(obj,
	                  (zend_objects_store_dtor_t)zend_objects_destroy_object,
	                  (zend_objects_free_object_storage_t)drizzle_con_obj_free,
	                   NULL TSRMLS_CC);
	obj->value.handlers= &drizzle_con_obj_handlers;

	return obj->value;
}

static zend_object_value
drizzle_con_obj_new(zend_class_entry *class_type TSRMLS_DC) {
	return drizzle_con_obj_new_ex(class_type, NULL TSRMLS_CC);
}

/*
 * Functions from con.h
 */

/* {{{ proto object drizzle_con_create(object drizzle)
   Initialize a connection structure. */
PHP_FUNCTION(drizzle_con_create) {
	zval *zobj;
	drizzle_obj *obj;
	drizzle_con_obj *con;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_con_ce);
	con= zend_object_store_get_object(return_value TSRMLS_CC);
	con->drizzle= zobj;
	Z_ADDREF_P(zobj);

	con->con= drizzle_con_create(&(obj->drizzle), NULL);
	if (con->con == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_FALSE;
	}

	con->flags|= DRIZZLE_CON_OBJ_CREATED;
	drizzle_con_set_context(con->con, con);
	drizzle_con_set_context_free_fn(con->con, drizzle_con_obj_context_free);
}
/* }}} */

/* {{{ proto object drizzle_con_clone(obeject drizzle_con)
   Clone a connection structure. */
PHP_FUNCTION(drizzle_con_clone) {
	zval *zobj;
	drizzle_obj *obj;
	zval *zcon;
	drizzle_con_obj *con;
	drizzle_con_obj *new;

	DRIZZLE_ZPMP(RETURN_NULL(), "O", &zobj, drizzle_ce, &zcon, drizzle_con_ce)

	con= zend_object_store_get_object(zcon TSRMLS_CC);

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_con_ce);
	new= zend_object_store_get_object(return_value TSRMLS_CC);
	new->drizzle= zobj;
	Z_ADDREF_P(zobj);

	new->con= drizzle_con_clone(&(obj->drizzle), NULL, con->con);
	if (new->con == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_FALSE;
	}

	new->flags|= DRIZZLE_CON_OBJ_CREATED;
	drizzle_con_set_context(new->con, new);
	drizzle_con_set_context_free_fn(new->con, drizzle_con_obj_context_free);
}
/* }}} */

/* {{{ proto void drizzle_con_free(object drizzle_con)
   Free a connection structure. */
PHP_FUNCTION(drizzle_con_free) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPP(RETURN_NULL(), "", &zobj, drizzle_con_ce)
	obj= zend_object_store_get_object(zobj TSRMLS_CC);

	if (obj->flags & DRIZZLE_CON_OBJ_CREATED) {
		drizzle_con_set_context_free_fn(obj->con, NULL);
		drizzle_con_free(obj->con);
		obj->flags&= ~DRIZZLE_CON_OBJ_CREATED;
	}
}
/* }}} */

/* {{{ proto int drizzle_con_fd(object drizzle_con)
   Get file descriptor for connection. */
PHP_FUNCTION(drizzle_con_fd) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)
	obj= zend_object_store_get_object(zobj TSRMLS_CC);

	RETURN_LONG(drizzle_con_fd(obj->con))
}
/* }}} */

/* {{{ proto long drizzle_con_set_fd(object drizzle_con, int fd)
   Use given file descriptor for connction. */
PHP_FUNCTION(drizzle_con_set_fd) {
	zval *zobj;
	drizzle_con_obj *obj;
	long fd;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_con_ce, &fd)

	obj->ret= drizzle_con_set_fd(obj->con, fd);
	if (obj->ret != DRIZZLE_RETURN_OK) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_con_error(obj->con));
        RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void drizzle_con_close(object drizzle_con)
   Close a connection. */
PHP_FUNCTION(drizzle_con_close) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	drizzle_con_close(obj->con);
}
/* }}} */

/* {{{ proto int drizzle_con_wait(object drizzle_con)
   Wait for I/O on connections. */
PHP_FUNCTION(drizzle_con_wait) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	obj->ret= drizzle_con_wait(&(obj->drizzle));
	if (obj->ret != DRIZZLE_RETURN_OK) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_error(&(obj->drizzle)));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto object drizzle_con_ready(object drizzle)
   Get next connection that is ready for I/O. */
PHP_FUNCTION(drizzle_con_ready) {
	zval *zobj;
	drizzle_obj *obj;
	drizzle_con_st *con;
	drizzle_con_obj *con_obj;
	drizzle_con_obj *new_obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	con= drizzle_con_ready(&(obj->drizzle));
	if (con == NULL) {
		RETURN_FALSE;
	}

	con_obj= drizzle_con_context(con);

	if (con_obj->flags & DRIZZLE_CON_OBJ_DEAD) {
		Z_TYPE_P(return_value)= IS_OBJECT;
		object_init_ex(return_value, drizzle_con_ce);
		new_obj= zend_object_store_get_object(return_value TSRMLS_CC);
		new_obj->drizzle= zobj;
		Z_ADDREF_P(zobj);
		new_obj->data= con_obj->data;

		new_obj->con= con;
		new_obj->flags|= DRIZZLE_CON_OBJ_CREATED;
		drizzle_con_set_context(new_obj->con, new_obj);

		efree(con_obj);
	} else {
		Z_TYPE_P(return_value)= IS_OBJECT;
		Z_OBJVAL_P(return_value)= con_obj->value;
		zval_copy_ctor(return_value);
	}
}
/* }}} */

/* {{{ proto object drizzle_con_drizzle(object drizzle_con)
   Get the drizzle_st struct that the connection belongs to. */
PHP_FUNCTION(drizzle_con_drizzle) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	*return_value= *(obj->drizzle);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto string drizzle_con_error(object drizzle_con)
   Return an error string for last library error encountered. */
PHP_FUNCTION(drizzle_con_error) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_error(obj->con), 1);
}
/* }}} */

/* {{{ proto int drizzle_con_errno(object drizzle_con)
   Value of errno in the case of a DRIZZLE_RETURN_ERRNO return value. */
PHP_FUNCTION(drizzle_con_errno) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_errno(obj->con));
}
/* }}} */

/* {{{ proto int drizzle_con_error_code(object drizzle_con)
   Get server defined error code for the last result read. */
PHP_FUNCTION(drizzle_con_error_code) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_error_code(obj->con));
}
/* }}} */

/* {{{ proto string drizzle_con_sqlstate(object drizzle_con)
   Get SQL state code for the last result read. */
PHP_FUNCTION(drizzle_con_sqlstate) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_sqlstate(obj->con), 1);
}
/* }}} */

/* {{{ proto int drizzle_con_options(object drizzle_con)
   Get options for a connection. */
PHP_FUNCTION(drizzle_con_options) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_options(obj->con));
}
/* }}} */

/* {{{ proto void drizzle_con_set_options(object dirzzle_con, int options)
   Set options for a connection. */
PHP_FUNCTION(drizzle_con_set_options) {
	zval *zobj;
	drizzle_con_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_con_ce, &options)

	drizzle_con_set_options(obj->con, DRIZZLE_CON_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_con_add_options(object drizzle_con, int options)
   Add options for a connection. */
PHP_FUNCTION(drizzle_con_add_options) {
	zval *zobj;
	drizzle_con_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_con_ce, &options)

	drizzle_con_add_options(obj->con, DRIZZLE_CON_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_con_remove_options(object drizzle_con, int options)
   Remove options for a connection. */
PHP_FUNCTION(drizzle_con_remove_options) {
	zval *zobj;
	drizzle_con_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_con_ce, &options)

	drizzle_con_remove_options(obj->con, DRIZZLE_CON_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto string drizzle_con_host(object drizzle_con)\
   Get TCP host for a connection. */
PHP_FUNCTION(drizzle_con_host) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_host(obj->con), 1);
}
/* }}} */

/* {{{ proto int drizzle_con_port(object drizzle_con)
   Get TCP port for a connection. */
PHP_FUNCTION(drizzle_con_port) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_port(obj->con));
}
/* }}} */

/* {{{ proto void drizzle_con_set_tcp(object drizzle_con, string host, int port)
   Set TCP host and port for a connection. */
PHP_FUNCTION(drizzle_con_set_tcp) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *host;
	int host_len;
	long port;

	DRIZZLE_ZPMP(RETURN_NULL(), "sl", &zobj, drizzle_con_ce, 
	             &host, &host_len, &port)

	drizzle_con_set_tcp(obj->con, strcmp(host, "") ? host : NULL, (in_port_t)port);
}
/* }}} */

/* {{{ proto string drizzle_con_uds(object drizzle_con)
   Get unix domain socket for a connection. */
PHP_FUNCTION(drizzle_con_uds) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_uds(obj->con), 1);
}
/* }}} */

/* {{{ proto drizzle_con_set_uds(object drizzle_con, string uds)
   Set unix domain socket for a connection. */
PHP_FUNCTION(drizzle_con_set_uds) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *uds;
	int uds_len;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_con_ce, &uds, &uds_len)

	drizzle_con_set_uds(obj->con, uds);
}
/* }}} */

/* {{{ proto string drizzle_con_user(object drizzle_con)
   Get username for a connection. */
PHP_FUNCTION(drizzle_con_user) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_user(obj->con), 1);
}
/* }}} */

/* {{{ proto string drizzle_con_password(object drizzle_con)
   Get password for a connection. */
PHP_FUNCTION(drizzle_con_password) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_password(obj->con), 1);
}
/* }}} */

/* {{{ proto void drizzle_con_set_auth(object drizzle_con, string user, string password)
   Set username and password for a connection. */
PHP_FUNCTION(drizzle_con_set_auth) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *user;
	int user_len;
	char *password;
	int password_len;

	DRIZZLE_ZPMP(RETURN_NULL(), "ss", &zobj, drizzle_con_ce, 
	             &user, &user_len, &password, &password_len)

	drizzle_con_set_auth(obj->con, user, password);
}
/* }}} */

/* {{{ proto string drizzle_con_db(object drizzle_con)
   Get database for a connection. */
PHP_FUNCTION(drizzle_con_db) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_db(obj->con), 1);
}
/* }}} */

/* {{{ proto bool drizzle_con_set_db(object drizzle_con, string db)
   Set database for a connection.  */
PHP_FUNCTION(drizzle_con_set_db) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *db;
	int db_len;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_con_ce, &db, &db_len)

	drizzle_con_set_db(obj->con, db);

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void drizzle_con_context(object drizzle_con)
   Get application data pointer for a connection. */
PHP_FUNCTION(drizzle_con_context) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	*return_value= *(obj->data);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto void drizzle con_set_context(object drizzle_con)
   Set application data pointer for a connection. */
PHP_FUNCTION(drizzle_con_set_context) {
	zval *zobj;
	drizzle_con_obj *obj;
	zval *zdata;

	DRIZZLE_ZPMP(RETURN_NULL(), "z", &zobj, drizzle_con_ce, &zdata)

	DRIZZLE_ZVAL_DONE(obj->data)
	obj->data= zdata;
	Z_ADDREF_P(zdata);
}
/* }}} */

/* {{{ proto int drizzle_con_protocol_version(object drizzle_con)
   Get protocol version for a connection. */
PHP_FUNCTION(drizzle_con_protocol_version) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_protocol_version(obj->con));
}
/* }}} */

/* {{{ proto string drizzle_con_server_version(object drizzle_con)
   drizzle_con_server_version */
PHP_FUNCTION(drizzle_con_server_version) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_server_version(obj->con), 1);
}
/* }}} */

/* {{{ proto int drizzle_con_server_version_number(object drizzle_con)
   Get server version number for a connection. */
PHP_FUNCTION(drizzle_con_server_version_number) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_server_version_number(obj->con));
}
/* }}} */

/* {{{ proto int drizzle_con_thread_id(object drizzle_con)
   Get thread ID for a connection. */
PHP_FUNCTION(drizzle_con_thread_id) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_thread_id(obj->con));
}
/* }}} */

/* {{{ proto string drizzle_con_scramble(object drizzle_con)
   Get scramble buffer for a connection. */
PHP_FUNCTION(drizzle_con_scramble) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_STRING((char *)drizzle_con_scramble(obj->con), 1);
}
/* }}} */

/* {{{ proto int drizzle_con_capabilities(object drizzle_con)
   Get capabilities for a connection. */
PHP_FUNCTION(drizzle_con_capabilities) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_capabilities(obj->con));
}
/* }}} */

/* {{{ proto int drizzle_con_charset(object drizzle_con)
   Get charset for a connection. */
PHP_FUNCTION(drizzle_con_charset) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_charset(obj->con));
}
/* }}} */

/* {{{ proto int drizzle_con_status(object drizzle_con)
   Get status for a connection. */
PHP_FUNCTION(drizzle_con_status) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_status(obj->con));
}
/* }}} */

/* {{{ proto int drizzle_con_max_packet_size(object drizzle_con)
   Get max packet size for a connection. */
PHP_FUNCTION(drizzle_con_max_packet_size) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(drizzle_con_max_packet_size(obj->con));
}
/* }}} */

/* {{{ proto void drizzle_return_code(object drizzle)
   Get the last return result for object */
PHP_FUNCTION(drizzle_con_return_code) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	RETURN_LONG(obj->ret);
}
/* }}} */

/*
 * Functions from con_client.h
 */

/* {{{ proto object drizzle_con_add_tcp(object drizzle, string host, int port, string user, string password, string db, int options)
   Add TCP (IPv4 or IPv6) connection with common arguments. */
PHP_FUNCTION(drizzle_con_add_tcp) {
	zval *zobj;
	drizzle_obj *obj;
	char *host;
	int host_len;
	long port;
	char *user;
	int user_len;
	char *password;
	int password_len;
	char *db;
	int db_len;
	long options;
	drizzle_con_obj *con;

	DRIZZLE_ZPMP(RETURN_NULL(), "slsssl", &zobj, drizzle_ce, 
	             &host, &host_len, &port, &user, &user_len, 
				 &password, &password_len, &db, &db_len, &options)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_con_ce);
	con= zend_object_store_get_object(return_value TSRMLS_CC);
	con->drizzle= zobj;
	Z_ADDREF_P(zobj);

	con->con= drizzle_con_add_tcp(&(obj->drizzle), NULL,
		strcmp(host, "") ? host : NULL, port, user, password, db, options);
	if (con->con == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	con->flags|= DRIZZLE_CON_OBJ_CREATED;
	drizzle_con_set_context(con->con, con);
	drizzle_con_set_context_free_fn(con->con, drizzle_con_obj_context_free);
}
/* }}} */

/* {{{ proto object drizzle_con_add_uds(object drizzle, string uds, string user, string password, string db, int options)
   Add Unix Domain Sockets connection with common arguments. */
PHP_FUNCTION(drizzle_con_add_uds) {
	zval *zobj;
	drizzle_obj *obj;
	char *uds;
	int uds_len;
	char *user;
	int user_len;
	char *password;
	int password_len;
	char *db;
	int db_len;
	long options;
	drizzle_con_obj *con;

	DRIZZLE_ZPMP(RETURN_NULL(), "ssssl", &zobj, drizzle_ce, 
	             &uds, &uds_len, &user, &user_len, &password, 
				 &password_len, &db, &db_len, &options)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_con_ce);
	con= zend_object_store_get_object(return_value TSRMLS_CC);
	con->drizzle= zobj;
	Z_ADDREF_P(zobj);

	con->con= drizzle_con_add_uds(&(obj->drizzle), con->con, uds, 
	                              user, password, db, options);
	if (con->con == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL()
	}

	con->flags|= DRIZZLE_CON_OBJ_CREATED;
	drizzle_con_set_context(con->con, con);
	drizzle_con_set_context_free_fn(con->con, drizzle_con_obj_context_free);
}
/* }}} */

/* {{{ proto int drizzle_con_connect(object drizzle_con)
   Connect to server. */
PHP_FUNCTION(drizzle_con_connect) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	obj->ret= drizzle_con_connect(obj->con);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_con_error(obj->con));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/*
 * Functions from handshake_client.h
 */

/* {{{ proto int drizzle_handshake_server_read(object drizzle_con)
   Read server handshake packet.  */
PHP_FUNCTION(drizzle_handshake_server_read) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	obj->ret= drizzle_handshake_server_read(obj->con);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_con_error(obj->con));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto int drizzle_handshake_client_write(object drizzle_con)
   Write client handshake packet. */
PHP_FUNCTION(drizzle_handshake_client_write) {
	zval *zobj;
	drizzle_con_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	obj->ret= drizzle_handshake_client_write(obj->con);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_con_error(obj->con));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/*
 * Functions from command_client.h
 */

/* {{{ proto object drizzle_quit(object drizzle_con)
   Send quit command on a connection. */
PHP_FUNCTION(drizzle_quit) {
	zval *zobj;
	drizzle_con_obj *obj;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_quit(obj->con, NULL, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_select_db(object drizzle_con, string db)
   Select a new default database for a connection. */
PHP_FUNCTION(drizzle_select_db) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *db;
	int db_len;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_con_ce, &db, &db_len)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_select_db(obj->con, NULL, db, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_shutdown(object drizzle_con, int level)
   Send a shutdown message at the given level. */
PHP_FUNCTION(drizzle_shutdown) {
	zval *zobj;
	drizzle_con_obj *obj;
	long level;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_con_ce, &level)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_shutdown(obj->con, NULL, level, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_ping(object drizzle_con)
   Send ping request on a connection. */
PHP_FUNCTION(drizzle_ping) {
	zval *zobj;
	drizzle_con_obj *obj;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_ping(obj->con, NULL, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_con_command_write(object drizzle_con, int command, string data, int total)
   Send raw command to server, possibly in parts. */
PHP_FUNCTION(drizzle_con_command_write) {
	zval *zobj;
	drizzle_con_obj *obj;
	long command;
	char *data;
	int data_len;
	long total;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "lsl", &zobj, drizzle_con_ce, 
	             &command, &data, &data_len, &total);

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_con_command_write(obj->con, NULL, command,
	                                      (uint8_t *)data, (size_t)data_len,
	                                      (size_t)total, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/*
 * Methods for drizzle_query.
 */

void drizzle_query_obj_context_free(drizzle_query_st *query, void *data) {
	drizzle_query_obj *obj= (drizzle_query_obj *)data;
    TSRMLS_FETCH();

	DRIZZLE_ZVAL_DONE(obj->zquery)
	DRIZZLE_ZVAL_DONE(obj->data)
	efree(obj);
}

PHP_METHOD(DrizzleQuery, __construct) {
	zval *zobj;
	drizzle_query_obj *obj;
	zval *zdrizzle;
	drizzle_obj *drizzle;

	DRIZZLE_ZPMP(DRIZZLE_EXCEPTION("A drizzle object is required", 0), "O", 
							&zobj, drizzle_query_ce, &zdrizzle, drizzle_ce)

	drizzle= zend_object_store_get_object(zdrizzle TSRMLS_CC);

	obj->drizzle= zdrizzle;
	Z_ADDREF_P(zdrizzle);

	obj->query= drizzle_query_create(&(drizzle->drizzle), NULL);
	if (obj->query == NULL) {
		DRIZZLE_EXCEPTION("Memory allocation failure.", 0);
	}

	obj->flags|= DRIZZLE_QUERY_OBJ_CREATED;
	drizzle_query_set_context(obj->query, obj);
	drizzle_query_set_context_free_fn(obj->query, drizzle_query_obj_context_free);
}

static void drizzle_query_obj_free(void *object TSRMLS_DC) {
	drizzle_query_obj *obj= (drizzle_query_obj *)object;

	/* We don't call drizzle_query_free here since the drizzle object can still
	   use the query internally. */

	DRIZZLE_ZVAL_DONE(obj->drizzle)

	zend_object_std_dtor(&(obj->std) TSRMLS_CC);

	if (obj->flags & DRIZZLE_QUERY_OBJ_CREATED) {
		obj->flags|= DRIZZLE_QUERY_OBJ_DEAD;
	} else {
		DRIZZLE_ZVAL_DONE(obj->zquery)
		DRIZZLE_ZVAL_DONE(obj->data)
		efree(object);
	}
}

static inline zend_object_value
drizzle_query_obj_new_ex(zend_class_entry *class_type,
                          drizzle_query_obj **query_ptr TSRMLS_DC) {
	drizzle_query_obj *obj;
	zval *tmp;

	obj= emalloc(sizeof(drizzle_query_obj));
	memset(obj, 0, sizeof(drizzle_query_obj));

	if (query_ptr) {
		*query_ptr= obj;
	}

	zend_object_std_init(&(obj->std), class_type TSRMLS_CC);
#if PHP_VERSION_ID < 50399
	zend_hash_copy(obj->std.properties, &(class_type->default_properties),
	               (copy_ctor_func_t)zval_add_ref, (void *)(&tmp),
	               sizeof(zval *));
#else
	object_properties_init(&(obj->std), class_type);
#endif

	obj->value.handle= zend_objects_store_put(obj,
	               (zend_objects_store_dtor_t)zend_objects_destroy_object,
	               (zend_objects_free_object_storage_t)drizzle_query_obj_free,
	                NULL TSRMLS_CC);
	obj->value.handlers= &drizzle_query_obj_handlers;

	return obj->value;
}

static zend_object_value
drizzle_query_obj_new(zend_class_entry *class_type TSRMLS_DC) {
	return drizzle_query_obj_new_ex(class_type, NULL TSRMLS_CC);
}

/*
 * Functions from query.h
 */

/* {{{ proto object drizzle_query(object drizzle_con, string query)
   Send query to server. */
PHP_FUNCTION(drizzle_query) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *query;
	int query_len;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_con_ce, &query, &query_len)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_query(obj->con, NULL, query, query_len, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_query_inc(object drizzle_con, string query, int total)
   Send query incrementally. */
PHP_FUNCTION(drizzle_query_inc) {
	zval *zobj;
	drizzle_con_obj *obj;
	char *query;
	int query_len;
	long total;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "sl", &zobj, drizzle_con_ce, 
	             &query, &query_len, &total)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_query_inc(obj->con, NULL, query, 
	                                  query_len, total, &obj->ret);

	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_query_add(object drizzle, zval zcon, object drizzle_con, zval zquery, int options, zval data)
   Add a query to be run concurrently. */
PHP_FUNCTION(drizzle_query_add) {
	zval *zobj;
	drizzle_obj *obj;
	zval *zcon;
	drizzle_con_obj *con;
	zval *zquery;
	long options;
	zval *data;
	drizzle_query_obj *query;

	DRIZZLE_ZPMP(RETURN_NULL(), "Ozlz", &zobj, drizzle_ce, &zcon, drizzle_con_ce,
	             &zquery, &options, &data)

	con= zend_object_store_get_object(zcon TSRMLS_CC);

	if (Z_TYPE_P(zquery) != IS_STRING) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Query must be a string.");
		RETURN_NULL();
	}

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_query_ce);
	query= zend_object_store_get_object(return_value TSRMLS_CC);
	query->drizzle= zobj;
	Z_ADDREF_P(zobj);
	query->zquery= zquery;
	Z_ADDREF_P(zquery);
	query->data= data;
	Z_ADDREF_P(data);

	query->query= drizzle_query_add(&(obj->drizzle), NULL, con->con, NULL,
	                                Z_STRVAL_P(zquery), Z_STRLEN_P(zquery),
	                                options, query);
	if (query->query == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	query->flags|= DRIZZLE_QUERY_OBJ_CREATED;
	drizzle_query_set_context_free_fn(query->query, drizzle_query_obj_context_free);
}
/* }}} */

/* {{{ proto object drizzle_query_create(object drizzle)
   Initialize a query structure. */
PHP_FUNCTION(drizzle_query_create) {
	zval *zobj;
	drizzle_obj *obj;
	drizzle_query_obj *query;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_query_ce);
	query= zend_object_store_get_object(return_value TSRMLS_CC);
	query->drizzle= zobj;
	Z_ADDREF_P(zobj);

	query->query= drizzle_query_create(&(obj->drizzle), NULL);
	if (query->query == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	query->flags|= DRIZZLE_QUERY_OBJ_CREATED;
	drizzle_query_set_context(query->query, query);
	drizzle_query_set_context_free_fn(query->query, drizzle_query_obj_context_free);
}
/* }}} */

/* {{{ proto void drizzle_query_free(object drizzle_query)
   Free a query structure. */
PHP_FUNCTION(drizzle_query_free) {
	zval *zobj;
	drizzle_query_obj *obj;

	DRIZZLE_ZPP(RETURN_NULL(), "", &zobj, drizzle_query_ce)
	obj= zend_object_store_get_object(zobj TSRMLS_CC);

	if (obj->flags & DRIZZLE_QUERY_OBJ_CREATED) {
		drizzle_query_set_context_free_fn(obj->query, NULL);
		drizzle_query_free(obj->query);
		obj->flags&= ~DRIZZLE_QUERY_OBJ_CREATED;
	}
}
/* }}} */

/* {{{ proto object drizzle_query_con(object drizzle_query)
   Get connection struct for a query. */
PHP_FUNCTION(drizzle_query_con) {
	zval *zobj;
	drizzle_query_obj *obj;
	drizzle_con_st *con;
	drizzle_con_obj *con_obj;
	drizzle_con_obj *new_obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	con= drizzle_query_con(obj->query);
	if (con == NULL) {
		RETURN_NULL();
	}

	con_obj= drizzle_con_context(con);

	if (con_obj->flags & DRIZZLE_CON_OBJ_DEAD) {
		Z_TYPE_P(return_value)= IS_OBJECT;
		object_init_ex(return_value, drizzle_con_ce);
		new_obj= zend_object_store_get_object(return_value TSRMLS_CC);
		new_obj->drizzle= obj->drizzle;
		Z_ADDREF_P(obj->drizzle);
		new_obj->data= con_obj->data;

		new_obj->con= con;
		new_obj->flags|= DRIZZLE_CON_OBJ_CREATED;
		drizzle_con_set_context(new_obj->con, new_obj);

		efree(con_obj);
	} else {
		Z_TYPE_P(return_value)= IS_OBJECT;
		Z_OBJVAL_P(return_value)= con_obj->value;
		zval_copy_ctor(return_value);
	}
}
/* }}} */

/* {{{ proto void drizzle_query_set_con(object drizzle_query, zval zcon, object drizzle_con_ce)
    Set connection struct for a query. */ /* XXX */
PHP_FUNCTION(drizzle_query_set_con) {
	zval *zobj;
	drizzle_query_obj *obj;
	zval *zcon;
	drizzle_con_obj *con;

	DRIZZLE_ZPMP(RETURN_NULL(), "O", &zobj, drizzle_query_ce, &zcon,
	             drizzle_con_ce)
	con= zend_object_store_get_object(zcon TSRMLS_CC);

	drizzle_query_set_con(obj->query, con->con);
}
/* }}} */

/* {{{ proto object drizzle_query_result(object drizzle_query)
   Get result struct for a query. */
PHP_FUNCTION(drizzle_query_result) {
	zval *zobj;
	drizzle_query_obj *obj;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_query_result(obj->query);
	if (result->result == NULL) {
		zval_dtor(return_value);
		RETURN_NULL()
	}

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto string drizzle_query_string(object drizzle_query)
   Get query string for a query. */
PHP_FUNCTION(drizzle_query_string) {
	zval *zobj;
	drizzle_query_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	*return_value= *(obj->zquery);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto void drizzle_query_set_string(object drizzle_query, string query)
   Set query string for a query. */
PHP_FUNCTION(drizzle_query_set_string) {
	zval *zobj;
	drizzle_query_obj *obj;
	zval *zquery;

	DRIZZLE_ZPMP(RETURN_NULL(), "z", &zobj, drizzle_query_ce, &zquery)

	DRIZZLE_ZVAL_DONE(obj->zquery)
	obj->zquery= zquery;
	Z_ADDREF_P(zquery);
}
/* }}} */

/* {{{ proto int drizzle_query_options(object drizzle_query)
   Get options for a query. */
PHP_FUNCTION(drizzle_query_options) {
	zval *zobj;
	drizzle_query_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	RETURN_LONG(drizzle_query_options(obj->query))
}
/* }}} */

/* {{{ proto void drizzle_query_set_options(object drizzle_query, int options)
   Set options for a query. */
PHP_FUNCTION(drizzle_query_set_options) {
	zval *zobj;
	drizzle_query_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_query_ce, &options)

	drizzle_query_set_options(obj->query, DRIZZLE_QUERY_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_query_add_options(object drizzle_query, int options)
   Add options for a query. */
PHP_FUNCTION(drizzle_query_add_options) {
	zval *zobj;
	drizzle_query_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_query_ce, &options)

	drizzle_query_add_options(obj->query, DRIZZLE_QUERY_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto void drizzle_query_remove_options(object drizzle_query, int options)
   Remove options for a query. */
PHP_FUNCTION(drizzle_query_remove_options) {
	zval *zobj;
	drizzle_query_obj *obj;
	long options;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_query_ce, &options)

	drizzle_query_remove_options(obj->query, DRIZZLE_QUERY_VALIDATE_OPTIONS(options));
}
/* }}} */

/* {{{ proto string drizzle_query_context(object drizzle_query)
   Get application data for a query. */
PHP_FUNCTION(drizzle_query_context) {
	zval *zobj;
	drizzle_query_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	*return_value= *(obj->data);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto string drizzle_query_set_context(object drizzle_query, string data)
   Set application data for a query. */
PHP_FUNCTION(drizzle_query_set_context) {
	zval *zobj;
	drizzle_query_obj *obj;
	zval *data;

	DRIZZLE_ZPMP(RETURN_NULL(), "z", &zobj, drizzle_query_ce, &data)

	DRIZZLE_ZVAL_DONE(obj->data)
	obj->data= data;
	Z_ADDREF_P(data);
}
/* }}} */

/* {{{ proto object drizzle_query_run(object drizzle)
   Run queries concurrently, returning when one is complete. */
PHP_FUNCTION(drizzle_query_run) {
	zval *zobj;
	drizzle_obj *obj;
	drizzle_query_st *query;
	drizzle_query_obj *query_obj;
	drizzle_query_obj *new_obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	query= drizzle_query_run(&(obj->drizzle), &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK &&
		obj->ret != DRIZZLE_RETURN_ERROR_CODE &&
		obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_error(&(obj->drizzle)));
		RETURN_FALSE;
	}

	if (query == NULL) {
		RETURN_NULL();
	} else {
		query_obj= drizzle_query_context(query);

		if (query_obj->flags & DRIZZLE_CON_OBJ_DEAD) {
			Z_TYPE_P(return_value)= IS_OBJECT;
			object_init_ex(return_value, drizzle_query_ce);
			new_obj= zend_object_store_get_object(return_value TSRMLS_CC);
			new_obj->drizzle= zobj;
			Z_ADDREF_P(zobj);
			new_obj->zquery= query_obj->zquery;
			new_obj->data= query_obj->data;

			new_obj->query= query;
			new_obj->flags|= DRIZZLE_QUERY_OBJ_CREATED;
			drizzle_query_set_context(new_obj->query, new_obj);

			efree(query_obj);
		} else {
			Z_TYPE_P(return_value)= IS_OBJECT;
			Z_OBJVAL_P(return_value)= query_obj->value;
			zval_copy_ctor(return_value);
		}
	}
}
/* }}} */

/* {{{ proto int drizzle_query_run_all(object drizzle)
   Run queries until they are all complete. */
PHP_FUNCTION(drizzle_query_run_all) {
	zval *zobj;
	drizzle_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_ce)

	obj->ret= drizzle_query_run_all(&(obj->drizzle));
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "%s", drizzle_error(&(obj->drizzle)));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void drizzle_return_code(object drizzle)
   Get the last return result for object */
PHP_FUNCTION(drizzle_query_return_code) {
	zval *zobj;
	drizzle_query_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_query_ce)

	RETURN_LONG(obj->ret);
}
/* }}} */

/* {{{ proto string drizzle_escape_string(object drizzle, string str)
   Escape a string or encode a string in hexadecimal. */
PHP_FUNCTION(drizzle_escape_string) {
	zval *zobj;
	drizzle_obj *obj;
	char *str;
	int str_len;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_ce, &str, &str_len)

	Z_TYPE_P(return_value)= IS_STRING;
	Z_STRVAL_P(return_value)= emalloc((str_len * 2) + 1);
	Z_STRLEN_P(return_value)= drizzle_escape_string(Z_STRVAL_P(return_value), 
	                                                str, str_len);
}
/* }}} */

/* {{{ proto string drizzle_hex_string(object drizzle, string str)
   Escape a string or encode a string in hexadecimal. */
PHP_FUNCTION(drizzle_hex_string) {
	zval *zobj;
	drizzle_obj *obj;
	char *str;
	int str_len;

	DRIZZLE_ZPMP(RETURN_NULL(), "s", &zobj, drizzle_ce, &str, &str_len)

	Z_TYPE_P(return_value)= IS_STRING;
	Z_STRVAL_P(return_value)= emalloc((str_len * 2) + 3);
	Z_STRLEN_P(return_value)= drizzle_hex_string(Z_STRVAL_P(return_value), 
	                                             str, str_len);
}
/* }}} */

/*
 * Methods for drizzle_result.
 */

PHP_METHOD(DrizzleResult, __construct) {
	zval *zobj;
	drizzle_result_obj *obj;
	zval *zcon;
	drizzle_con_obj *con;

	DRIZZLE_ZPMP(DRIZZLE_EXCEPTION("A drizzle object is required", 0), "O", 
							&zobj, drizzle_result_ce, &zcon, drizzle_con_ce)

	con= zend_object_store_get_object(zcon TSRMLS_CC);

	obj->drizzle= con->drizzle;
	Z_ADDREF_P(con->drizzle);

	obj->result= drizzle_result_create(con->con, NULL);
	if (obj->result == NULL) {
		DRIZZLE_EXCEPTION("Memory allocation failure.", 0);
	}

	obj->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}

static void drizzle_result_obj_free(void *object TSRMLS_DC) {
	drizzle_result_obj *result= (drizzle_result_obj *)object;

	if (result->flags & DRIZZLE_RESULT_OBJ_CREATED) {
		drizzle_result_free(result->result);
	}

	DRIZZLE_ZVAL_DONE(result->drizzle)
	DRIZZLE_ZVAL_DONE(result->field_sizes)
	zend_object_std_dtor(&(result->std) TSRMLS_CC);
	efree(object);
}

static inline zend_object_value
drizzle_result_obj_new_ex(zend_class_entry *class_type,
                          drizzle_result_obj **result_ptr TSRMLS_DC) {
	drizzle_result_obj *result;
	zend_object_value value;
	zval *tmp;

	result= emalloc(sizeof(drizzle_result_obj));
	memset(result, 0, sizeof(drizzle_result_obj));

	if (result_ptr) {
		*result_ptr= result;
	}

	zend_object_std_init(&(result->std), class_type TSRMLS_CC);
#if PHP_VERSION_ID < 50399
	zend_hash_copy(result->std.properties, &(class_type->default_properties),
	               (copy_ctor_func_t)zval_add_ref, (void *)(&tmp),
	               sizeof(zval *));
#else
	object_properties_init(&(result->std), class_type);
#endif
	value.handle= zend_objects_store_put(result,
	              (zend_objects_store_dtor_t)zend_objects_destroy_object,
	              (zend_objects_free_object_storage_t)drizzle_result_obj_free,
	               NULL TSRMLS_CC);
	value.handlers= &drizzle_result_obj_handlers;

	return value;
}

static zend_object_value
drizzle_result_obj_new(zend_class_entry *class_type TSRMLS_DC) {
	return drizzle_result_obj_new_ex(class_type, NULL TSRMLS_CC);
}

/*
 * Functions from result.h
 */

/* {{{ proto object drizzle_result_create(object drizzle)
   Initialize a result structure. */
PHP_FUNCTION(drizzle_result_create) {
	zval *zobj;
	drizzle_con_obj *obj;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	result->result= drizzle_result_create(obj->con, NULL);
	if (result->result == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto object drizzle_result_clone(object drizzle, object drizzle_result)
   Clone a connection structure. */
PHP_FUNCTION(drizzle_result_clone) {
	zval *zobj;
	drizzle_con_obj *obj;
	zval *zresult;
	drizzle_result_obj *result;
	drizzle_result_obj *new;

	DRIZZLE_ZPMP(RETURN_NULL(), "O", &zobj, drizzle_ce, &zresult,
	             drizzle_result_ce)

	result= zend_object_store_get_object(zresult TSRMLS_CC);

	DRIZZLE_RESULT_INIT(return_value, new, obj->drizzle)

	new->result= drizzle_result_clone(obj->con, NULL, result->result);
	if (new->result == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	new->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto void drizzle_result_free(object drizzle_result)
   Free a result structure. */
PHP_FUNCTION(drizzle_result_free) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	if (obj->flags & DRIZZLE_RESULT_OBJ_CREATED) {
		drizzle_result_free(obj->result);
		obj->flags&= ~DRIZZLE_RESULT_OBJ_CREATED;
	}
}
/* }}} */

/* {{{ proto object drizzle_result_drizzle_con(object drizzle_result)
   Get the drizzle_con_st struct that the result belongs to.  */
PHP_FUNCTION(drizzle_result_drizzle_con) {
	zval *zobj;
	drizzle_result_obj *obj;
	drizzle_con_st *con;
	drizzle_con_obj *con_obj;
	drizzle_con_obj *new_obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	con= drizzle_result_drizzle_con(obj->result);
	if (con == NULL) {
		RETURN_NULL();
	}

	con_obj= drizzle_con_context(con);

	if (con_obj->flags & DRIZZLE_CON_OBJ_DEAD) {
		Z_TYPE_P(return_value)= IS_OBJECT;
		object_init_ex(return_value, drizzle_con_ce);
		new_obj= zend_object_store_get_object(return_value TSRMLS_CC);
		new_obj->drizzle= obj->drizzle;
		Z_ADDREF_P(obj->drizzle);
		new_obj->data= con_obj->data;

		new_obj->con= con;
		new_obj->flags|= DRIZZLE_CON_OBJ_CREATED;
		drizzle_con_set_context(new_obj->con, new_obj);

		efree(con_obj);
	} else {
		Z_TYPE_P(return_value)= IS_OBJECT;
		Z_OBJVAL_P(return_value)= con_obj->value;
		zval_copy_ctor(return_value);
	}
}
/* }}} */

/* {{{ proto bool drizzle_result_eof(object drizzle_result)
   Get EOF flag for a result. */
PHP_FUNCTION(drizzle_result_eof) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_BOOL(drizzle_result_eof(obj->result));
}
/* }}} */

/* {{{ proto string drizzle_result_info(object drizzle_result)
   Get information string for a result. */
PHP_FUNCTION(drizzle_result_info) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_STRING((char *)drizzle_result_info(obj->result), 1);
}
/* }}} */

/* {{{ proto string drizzle_result_error(object drizzle_result)
   Get error string for a result. */
PHP_FUNCTION(drizzle_result_error) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_STRING((char *)drizzle_result_error(obj->result), 1);
}
/* }}} */

/* {{{ proto int drizzle_result_error_code(object dirzzle_result)
   Get server defined error code for a result. */
PHP_FUNCTION(drizzle_result_error_code) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_error_code(obj->result));
}
/* }}} */

/* {{{ proto string drizzle_result_sqlstate(object drizzle_result)
   Get SQL state code for a result. */
PHP_FUNCTION(drizzle_result_sqlstate) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_STRING((char *)drizzle_result_sqlstate(obj->result), 1);
}
/* }}} */

/* {{{ proto int drizzle_result_warning_count(object drizzle_result)
   Get the number of warnings encounted during a command. */
PHP_FUNCTION(drizzle_result_warning_count) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_warning_count(obj->result));
}
/* }}} */

/* {{{ proto int drizzle_result_insert_id(object drizzle_result)
   Get inet ID of the last command, if any. */
PHP_FUNCTION(drizzle_result_insert_id) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_insert_id(obj->result));
}
/* }}} */

/* {{{ proto int drizzle_result_affected_rows(object drizzle_result)
   Get the number of affected rows during the command. */
PHP_FUNCTION(drizzle_result_affected_rows) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_affected_rows(obj->result));
}
/* }}} */

/* {{{ proto int drizzle_result_column_count(object drizzle_result)
   Get the number of columns in a result set. */
PHP_FUNCTION(drizzle_result_column_count) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_column_count(obj->result));
}
/* }}} */

/* {{{ proto int drizzle_result_row_count(object drizzle_result)
   Get the number of rows returned for the command. */
PHP_FUNCTION(drizzle_result_row_count) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_row_count(obj->result));
}
/* }}} */

/*
 * Functions from result_client.h
 */

/* {{{ proto object drizzle_result_read(object drizzle_con)
   Read result packet. */
PHP_FUNCTION(drizzle_result_read) {
	zval *zobj;
	drizzle_con_obj *obj;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_con_ce)

	DRIZZLE_RESULT_INIT(return_value, result, obj->drizzle)

	(void)drizzle_result_read(obj->con, result->result, &obj->ret);
	DRIZZLE_RESULT_ERROR_CHECK(obj->ret, result->result, return_value, obj->con)

	result->flags|= DRIZZLE_RESULT_OBJ_CREATED;
}
/* }}} */

/* {{{ proto int drizzle_result_buffer(object drizzle_result)
   Buffer all data for a result. */
PHP_FUNCTION(drizzle_result_buffer) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	obj->ret= drizzle_result_buffer(obj->result);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto int drizzle_result_row_size(object drizzle_result)
   Get result row packet size. */
PHP_FUNCTION(drizzle_result_row_size) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_result_row_size(obj->result));
}
/* }}} */

/* {{{ proto void drizzle_return_code(object drizzle)
   Get the last return result for object */
PHP_FUNCTION(drizzle_result_return_code) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(obj->ret);
}
/* }}} */

/*
 * Methods for drizzle_column.
 */

PHP_METHOD(DrizzleColumn, __construct) {
	zval *zobj;
	drizzle_column_obj *obj;
	zval *zresult;
	drizzle_result_obj *result;

	DRIZZLE_ZPMP(DRIZZLE_EXCEPTION("A drizzle object is required", 0), "O", 
					&zobj, drizzle_column_ce, &zresult, drizzle_result_ce)

	result= zend_object_store_get_object(zresult TSRMLS_CC);

	obj->result= zresult;
	Z_ADDREF_P(zresult);

	obj->column= drizzle_column_create(result->result, NULL);
	if (obj->column == NULL) {
		DRIZZLE_EXCEPTION("Memory allocation failure.", 0);
	}

	obj->flags|= DRIZZLE_COLUMN_OBJ_CREATED;
}

static void drizzle_column_obj_free(void *object TSRMLS_DC) {
	drizzle_column_obj *obj= (drizzle_column_obj *)object;

	if (obj->flags & DRIZZLE_COLUMN_OBJ_CREATED) {
		drizzle_column_free(obj->column);
	}

	DRIZZLE_ZVAL_DONE(obj->result);
	zend_object_std_dtor(&(obj->std) TSRMLS_CC);
	efree(object);
}

static inline zend_object_value
drizzle_column_obj_new_ex(zend_class_entry *class_type,
                          drizzle_column_obj **column_ptr TSRMLS_DC) {
	drizzle_column_obj *obj;
	zend_object_value value;
	zval *tmp;

	obj= emalloc(sizeof(drizzle_column_obj));
	memset(obj, 0, sizeof(drizzle_column_obj));

	if (column_ptr) {
		*column_ptr= obj;
	}

	zend_object_std_init(&(obj->std), class_type TSRMLS_CC);
#if PHP_VERSION_ID < 50399
	zend_hash_copy(obj->std.properties, &(class_type->default_properties),
	               (copy_ctor_func_t)zval_add_ref, (void *)(&tmp),
	               sizeof(zval *));
#else
	object_properties_init(&(obj->std), class_type);
#endif

	value.handle= zend_objects_store_put(obj,
	             (zend_objects_store_dtor_t)zend_objects_destroy_object,
	             (zend_objects_free_object_storage_t)drizzle_column_obj_free,
	              NULL TSRMLS_CC);
	value.handlers= &drizzle_column_obj_handlers;

	return value;
}

static zend_object_value
drizzle_column_obj_new(zend_class_entry *class_type TSRMLS_DC) {
	return drizzle_column_obj_new_ex(class_type, NULL TSRMLS_CC);
}

/*
 * Functions from column.h
 */

/* {{{ proto object drizzle_column_create(object drizzle_result)
   Initialize a column structure. */
PHP_FUNCTION(drizzle_column_create) {
	zval *zobj;
	drizzle_result_obj *obj;
	drizzle_column_obj *column;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_column_ce);
	column= zend_object_store_get_object(return_value TSRMLS_CC);
	column->result= zobj;
	Z_ADDREF_P(zobj);

	column->column= drizzle_column_create(obj->result, NULL);
	if (column->column == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_NULL();
	}

	column->flags|= DRIZZLE_COLUMN_OBJ_CREATED;
}
/* }}} */

/* {{{ proto void drizzle_column_free(object drizzle_column)
   Free a column structure. */
PHP_FUNCTION(drizzle_column_free) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPP(RETURN_NULL(), "", &zobj, drizzle_column_ce)
	obj= zend_object_store_get_object(zobj TSRMLS_CC);

	if (obj->flags & DRIZZLE_COLUMN_OBJ_CREATED) {
		drizzle_column_free(obj->column);
		obj->flags&= ~DRIZZLE_COLUMN_OBJ_CREATED;
	}
}
/* }}} */

/* {{{ proto void drizzle_column_drizzle_result(object drizzle_column)
   Get the drizzle_result_st struct that the column belongs to. */
PHP_FUNCTION(drizzle_column_drizzle_result) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	*return_value= *(obj->result);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto string drizzle_column_catalog(object drizzle_column)
   Get catalog name for a column. */
PHP_FUNCTION(drizzle_column_catalog) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_catalog(obj->column), 1);
}
/* }}} */

/* {{{ proto string drizzle_column_db(object drizzle_column)
   Get database name for a column. */
PHP_FUNCTION(drizzle_column_db) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_db(obj->column), 1);
}
/* }}} */

/* {{{ proto string drizzle_column_table(object drizzle_column)
   Get table name for a column. */
PHP_FUNCTION(drizzle_column_table) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_table(obj->column), 1);
}
/* }}} */

/* {{{ proto string drizzle_column_orig_table(object drizzle_column)
   Get original table name for a column. */
PHP_FUNCTION(drizzle_column_orig_table) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_orig_table(obj->column), 1);
}
/* }}} */

/* {{{ proto string drizzle_column_name(object drizzle_column)
   Get column name for a column. */
PHP_FUNCTION(drizzle_column_name) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_name(obj->column), 1);
}
/* }}} */

/* {{{ proto string drizzle_column_orig_name(object drizzle_column)
   Get original column name for a column. */
PHP_FUNCTION(drizzle_column_orig_name) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_STRING((char *)drizzle_column_orig_name(obj->column), 1);
}
/* }}} */

/* {{{ proto int drizzle_column_charset(object drizzle_column)
   Get charset for a column. */
PHP_FUNCTION(drizzle_column_charset) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_charset(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_size(object drizzle_column)
   Get size of a column. */
PHP_FUNCTION(drizzle_column_size) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_size(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_max_size(object drizzle_column)
   Get max size of a column. */
PHP_FUNCTION(drizzle_column_max_size) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_max_size(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_type(object drizzle_column)
   drizzle_column_type */
PHP_FUNCTION(drizzle_column_type) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_type(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_type_drizzle(object drizzle_column)
   Get the Drizzle type of a column. */
PHP_FUNCTION(drizzle_column_type_drizzle) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_type_drizzle(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_flags(object drizzle_column)
   Get flags for a column. */
PHP_FUNCTION(drizzle_column_flags) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_flags(obj->column));
}
/* }}} */

/* {{{ proto int drizzle_column_decimals(object drizzle_column)
   Get the number of decimals for numeric columns. */
PHP_FUNCTION(drizzle_column_decimals) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(drizzle_column_decimals(obj->column));
}
/* }}} */

/* {{{ proto string drizzle_column_default_value(object drizzle_column)
   Get default value for a column. */
PHP_FUNCTION(drizzle_column_default_value) {
	zval *zobj;
	drizzle_column_obj *obj;
	char *default_value;
	size_t size;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	default_value= (char *)drizzle_column_default_value(obj->column, &size);

	RETURN_STRINGL(default_value, size, 1);
}
/* }}} */

/*
 * Functions from column_client.h
 */

/* {{{ proto int drizzle_column_skip(object drizzle_result)
   Skip all columns in result. */
PHP_FUNCTION(drizzle_column_skip) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	obj->ret= drizzle_column_skip(obj->result);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto object drizzle_column_read(object drizzle_result)
   Read column information. */
PHP_FUNCTION(drizzle_column_read) {
	zval *zobj;
	drizzle_result_obj *obj;
	drizzle_column_obj *column;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_column_ce);
	column= zend_object_store_get_object(return_value TSRMLS_CC);
	column->result= zobj;
	Z_ADDREF_P(zobj);

	column->column= drizzle_column_read(obj->result, NULL, &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, 
		                 "Memory allocation failure.");
		zval_dtor(return_value);
		RETURN_FALSE;
	}

	if (column->column == NULL) {
		zval_dtor(return_value);
		RETURN_NULL();
	}

	column->flags|= DRIZZLE_COLUMN_OBJ_CREATED;
}
/* }}} */

/* {{{ proto int drizzle_column_buffer(object drizzle_result)
   Buffer all columns in result structure. */
PHP_FUNCTION(drizzle_column_buffer) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	obj->ret= drizzle_column_buffer(obj->result);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto object drizzle_column_next(object drizzle_result)
   Get next buffered column from a result structure. */
PHP_FUNCTION(drizzle_column_next) {
	zval *zobj;
	drizzle_result_obj *obj;
	drizzle_column_obj *column;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_column_ce);
	column= zend_object_store_get_object(return_value TSRMLS_CC);
	column->result= zobj;
	Z_ADDREF_P(zobj);

	column->column= drizzle_column_next(obj->result);
	if (column->column == NULL) {
		zval_dtor(return_value);
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto object drizzle_column_prev(object drizzle_result)
   Get previous buffered column from a result structure. */
PHP_FUNCTION(drizzle_column_prev) {
	zval *zobj;
	drizzle_result_obj *obj;
	drizzle_column_obj *column;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_column_ce);
	column= zend_object_store_get_object(return_value TSRMLS_CC);
	column->result= zobj;
	Z_ADDREF_P(zobj);

	column->column= drizzle_column_prev(obj->result);
	if (column->column == NULL) {
		zval_dtor(return_value);
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto void drizzle_column_seek(object drizzle_result, int column_index)
   Seek to the given buffered column in a result structure. */
PHP_FUNCTION(drizzle_column_seek) {
	zval *zobj;
	drizzle_result_obj *obj;
	long column_index;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_result_ce, &column_index)

	drizzle_column_seek(obj->result, column_index);
}
/* }}} */

/* {{{ proto object drizzle_column_index(object drizzle_result, int column_index)
   Get the given buffered column from a result structure. */
PHP_FUNCTION(drizzle_column_index) {
	zval *zobj;
	drizzle_result_obj *obj;
	long column_index;
	drizzle_column_obj *column;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_result_ce, &column_index)

	Z_TYPE_P(return_value)= IS_OBJECT;
	object_init_ex(return_value, drizzle_column_ce);
	column= zend_object_store_get_object(return_value TSRMLS_CC);
	column->result= zobj;
	Z_ADDREF_P(zobj);

	column->column= drizzle_column_index(obj->result, column_index);
	if (column->column == NULL) {
		zval_dtor(return_value);
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto */
PHP_FUNCTION(drizzle_column_current) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	RETURN_LONG(drizzle_column_current(obj->result));
}
/* }}} */

/* {{{ proto void drizzle_return_code(object drizzle)
   Get the last return result for object */
PHP_FUNCTION(drizzle_column_return_code) {
	zval *zobj;
	drizzle_column_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_column_ce)

	RETURN_LONG(obj->ret);
}
/* }}} */

/*
 * Functions from row_client.h
 */

/* Macro to use in _buffer/_next/_prev/_index. */
#define DRIZZLE_ROW_COPY { \
	if (row == NULL) { \
		RETURN_NULL() \
	} \
	array_init(return_value); \
	field_sizes= drizzle_row_field_sizes(obj->result); \
	DRIZZLE_ZVAL_DONE(obj->field_sizes) \
	MAKE_STD_ZVAL(obj->field_sizes) \
	array_init(obj->field_sizes); \
	for (x= 0; x < drizzle_result_column_count(obj->result); x++) { \
		if (row[x] == NULL) { \
			add_next_index_null(return_value); \
		} else { \
			add_next_index_stringl(return_value, row[x], field_sizes[x], 1); \
		} \
		add_next_index_long(obj->field_sizes, field_sizes[x]); \
	} \
}

/* {{{ proto int drizzle_row_read(object drizzle_result)
   Get next row number for unbuffered results. */
PHP_FUNCTION(drizzle_row_read) {
	zval *zobj;
	drizzle_result_obj *obj;
	uint64_t row;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	row= drizzle_row_read(obj->result, &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_LONG(0);
	}

	RETURN_LONG(row);
}
/* }}} */

/* {{{ proto list drizzle_row_buffer(object drizzle_result)
   Read and buffer one row. */
PHP_FUNCTION(drizzle_row_buffer) {
	zval *zobj;
	drizzle_result_obj *obj;
	char **row;
	size_t *field_sizes;
	uint16_t x;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	row= (char **)drizzle_row_buffer(obj->result, &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	DRIZZLE_ROW_COPY

	drizzle_row_free(obj->result, (drizzle_row_t)row);
}
/* }}} */

/* {{{ proto int drizzle_row_field_sizes(object drizzle_result)
   Get an array of all field sizes for buffered rows. */
PHP_FUNCTION(drizzle_row_field_sizes) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	*return_value= *(obj->field_sizes);
	zval_copy_ctor(return_value);
    INIT_PZVAL(return_value);
}
/* }}} */

/* {{{ proto list drizzle_row_next(object drizzle_result)
   Get next buffered row from a fully buffered result. */
PHP_FUNCTION(drizzle_row_next) {
	zval *zobj;
	drizzle_result_obj *obj;
	char **row;
	size_t *field_sizes;
	uint16_t x;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	row= (char **)drizzle_row_next(obj->result);

	DRIZZLE_ROW_COPY
}
/* }}} */

/* {{{ proto list drizzle_row_prev(object drizzle_result)
   Get previous buffered row from a fully buffered result. */
PHP_FUNCTION(drizzle_row_prev) {
	zval *zobj;
	drizzle_result_obj *obj;
	char **row;
	size_t *field_sizes;
	uint16_t x;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	row= (char **)drizzle_row_prev(obj->result);

	DRIZZLE_ROW_COPY
}
/* }}} */

/* {{{ proto void drizzle_row_seek(object drizzle_result, int row_index)
   Seek to the given buffered row in a fully buffered result. */
PHP_FUNCTION(drizzle_row_seek) {
	zval *zobj;
	drizzle_result_obj *obj;
	long row_index;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_result_ce, &row_index)

	drizzle_row_seek(obj->result, row_index);
}
/* }}} */

/* {{{ proto list drizzle_row_index(object drizzle_result, int row_index)
   Get the given buffered row from a fully buffered result. */
PHP_FUNCTION(drizzle_row_index) {
	zval *zobj;
	drizzle_result_obj *obj;
	long row_index;
	char **row;
	size_t *field_sizes;
	uint16_t x;

	DRIZZLE_ZPMP(RETURN_NULL(), "l", &zobj, drizzle_result_ce, &row_index)

	row= (char **)drizzle_row_index(obj->result, row_index);

	DRIZZLE_ROW_COPY
}
/* }}} */

/* {{{ proto int drizzle_row_current(object drizzle_rexutl)
   Get current row number. */
PHP_FUNCTION(drizzle_row_current) {
	zval *zobj;
	drizzle_result_obj *obj;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce);

	RETURN_LONG(drizzle_row_current(obj->result));
}
/* }}} */

/*
 * Functions from field_client.h
 */

/* {{{ proto list drizzle_field_read(object drizzle_result)
   Read field for unbuffered result, possibly in parts. */
PHP_FUNCTION(drizzle_field_read) {
	zval *zobj;
	drizzle_result_obj *obj;
	char *field;
	size_t offset;
	size_t size;
	size_t total;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	field= (char *)drizzle_field_read(obj->result, &offset, &size, &total,
	                                  &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT &&
	    obj->ret != DRIZZLE_RETURN_ROW_END && obj->ret != DRIZZLE_RESULT_ROW_BREAK) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	array_init(return_value);
	/* XXX access via obj->returnCode()
	add_next_index_long(return_value, obj->ret); */
	if (field == NULL) {
		add_next_index_null(return_value);
	} else {
		add_next_index_stringl(return_value, field, size, 1);
	}
	add_next_index_long(return_value, offset);
	add_next_index_long(return_value, total);
}
/* }}} */


/* {{{ proto string drizzle_field_buffer(object drizzle_result)
   Buffer one field. */
PHP_FUNCTION(drizzle_field_buffer) {
	zval *zobj;
	drizzle_result_obj *obj;
	char *field;
	size_t total;

	DRIZZLE_ZPMP(RETURN_NULL(), "", &zobj, drizzle_result_ce)

	field= (char *)drizzle_field_buffer(obj->result, &total, &obj->ret);
	if (obj->ret != DRIZZLE_RETURN_OK && obj->ret != DRIZZLE_RETURN_IO_WAIT &&
	    obj->ret != DRIZZLE_RETURN_ROW_END && obj->ret != DRIZZLE_RESULT_ROW_BREAK) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", drizzle_con_error(
		                 drizzle_result_drizzle_con(obj->result)));
		RETURN_FALSE;
	}

	if (field == NULL) {
		RETURN_NULL();
	} else {
		RETURN_STRINGL(field, total, 1);
	}

	/* XXX will this get called? */
	drizzle_field_free((drizzle_field_t)field);
}
/* }}} */

/* {{{ proto void drizzle_field_free(object drizzle_field)
   Free a buffered field. */
#if jluedke_0
PHP_FUNCTION(drizzle_field_free) {
    RETURN_FALSE;
}
#endif
/* }}} */

/* Function list. */
zend_function_entry drizzle_functions[]= {
	/* Functions from drizzle.h */
	PHP_FE(drizzle_version, arginfo_drizzle_version)
	PHP_FE(drizzle_create, arginfo_drizzle_create)
	PHP_FE(drizzle_clone, arginfo_drizzle_clone)
	PHP_FE(drizzle_free, arginfo_drizzle_free)
	PHP_FE(drizzle_error, arginfo_drizzle_error)
	PHP_FE(drizzle_errno, arginfo_drizzle_errno)
	PHP_FE(drizzle_error_code, arginfo_drizzle_error_code)
	PHP_FE(drizzle_sqlstate, arginfo_drizzle_sqlstate)
	PHP_FE(drizzle_options, arginfo_drizzle_options)
	PHP_FE(drizzle_set_options, arginfo_drizzle_set_options)
	PHP_FE(drizzle_add_options, arginfo_drizzle_add_options)
	PHP_FE(drizzle_remove_options, arginfo_drizzle_remove_options)
	PHP_FE(drizzle_return_code, arginfo_drizzle_return_code)

	/* Functions from con.h */
	PHP_FE(drizzle_con_create, arginfo_drizzle_con_create)
	PHP_FE(drizzle_con_clone, arginfo_drizzle_con_clone)
	PHP_FE(drizzle_con_free, arginfo_drizzle_con_free)
	PHP_FE(drizzle_con_fd, arginfo_drizzle_con_fd)
	PHP_FE(drizzle_con_set_fd, arginfo_drizzle_con_set_fd)
	PHP_FE(drizzle_con_close, arginfo_drizzle_con_close)
	PHP_FE(drizzle_con_wait, arginfo_drizzle_con_wait)
	PHP_FE(drizzle_con_ready, arginfo_drizzle_con_ready)
	PHP_FE(drizzle_con_drizzle, arginfo_drizzle_con_drizzle)
	PHP_FE(drizzle_con_error, arginfo_drizzle_con_error)
	PHP_FE(drizzle_con_errno, arginfo_drizzle_con_errno)
	PHP_FE(drizzle_con_error_code, arginfo_drizzle_con_error_code)
	PHP_FE(drizzle_con_sqlstate, arginfo_drizzle_con_sqlstate)
	PHP_FE(drizzle_con_options, arginfo_drizzle_con_options)
	PHP_FE(drizzle_con_set_options, arginfo_drizzle_con_set_options)
	PHP_FE(drizzle_con_add_options, arginfo_drizzle_con_add_options)
	PHP_FE(drizzle_con_remove_options, arginfo_drizzle_con_remove_options)
	PHP_FE(drizzle_con_host, arginfo_drizzle_con_host)
	PHP_FE(drizzle_con_port, arginfo_drizzle_con_port)
	PHP_FE(drizzle_con_set_tcp, arginfo_drizzle_con_set_tcp)
	PHP_FE(drizzle_con_uds, arginfo_drizzle_con_uds)
	PHP_FE(drizzle_con_set_uds, arginfo_drizzle_con_set_uds)
	PHP_FE(drizzle_con_user, arginfo_drizzle_con_user)
	PHP_FE(drizzle_con_password, arginfo_drizzle_con_password)
	PHP_FE(drizzle_con_set_auth, arginfo_drizzle_con_set_auth)
	PHP_FE(drizzle_con_db, arginfo_drizzle_con_db)
	PHP_FE(drizzle_con_set_db, arginfo_drizzle_con_set_db)
	PHP_FE(drizzle_con_context, arginfo_drizzle_con_context)
	PHP_FE(drizzle_con_set_context, arginfo_drizzle_con_set_context)
	PHP_FE(drizzle_con_protocol_version, arginfo_drizzle_con_protocol_version)
	PHP_FE(drizzle_con_server_version, arginfo_drizzle_con_server_version)
	PHP_FE(drizzle_con_server_version_number, arginfo_drizzle_con_server_version_number)
	PHP_FE(drizzle_con_thread_id, arginfo_drizzle_con_thread_id)
	PHP_FE(drizzle_con_scramble, arginfo_drizzle_con_scramble)
	PHP_FE(drizzle_con_capabilities, arginfo_drizzle_con_capabilities)
	PHP_FE(drizzle_con_charset, arginfo_drizzle_con_charset)
	PHP_FE(drizzle_con_status, arginfo_drizzle_con_status)
	PHP_FE(drizzle_con_max_packet_size, arginfo_drizzle_con_max_packet_size)
	PHP_FE(drizzle_con_return_code, arginfo_drizzle_con_return_code)

	/* Functions from con_client.h */
	PHP_FE(drizzle_con_add_tcp, arginfo_drizzle_con_add_tcp)
	PHP_FE(drizzle_con_add_uds, arginfo_drizzle_con_add_uds)
	PHP_FE(drizzle_con_connect, arginfo_drizzle_con_connect)

	/* Functions from handshake_client.h */
	PHP_FE(drizzle_handshake_server_read, arginfo_drizzle_handshake_server_read)
	PHP_FE(drizzle_handshake_client_write, arginfo_drizzle_handshake_client_write)

	/* Functions from command_client.h */
	PHP_FE(drizzle_quit, arginfo_drizzle_quit)
	PHP_FE(drizzle_select_db, arginfo_drizzle_select_db)
	PHP_FE(drizzle_shutdown, arginfo_drizzle_shutdown)
	PHP_FE(drizzle_ping, arginfo_drizzle_ping)
	PHP_FE(drizzle_con_command_write, arginfo_drizzle_con_command_write)

	/* Functions from query.h */
	PHP_FE(drizzle_query, arginfo_drizzle_query)
	PHP_FE(drizzle_query_inc, arginfo_drizzle_query_inc)
	PHP_FE(drizzle_query_add, arginfo_drizzle_query_add)
	PHP_FE(drizzle_query_create, arginfo_drizzle_query_create)
	PHP_FE(drizzle_query_free, arginfo_drizzle_query_free)
	PHP_FE(drizzle_query_con, arginfo_drizzle_query_con)
	PHP_FE(drizzle_query_set_con, arginfo_drizzle_query_set_con)
	PHP_FE(drizzle_query_result, arginfo_drizzle_query_result)
	PHP_FE(drizzle_query_string, arginfo_drizzle_query_string)
	PHP_FE(drizzle_query_set_string, arginfo_drizzle_query_set_string)
	PHP_FE(drizzle_query_options, arginfo_drizzle_query_options)
	PHP_FE(drizzle_query_set_options, arginfo_drizzle_query_set_options)
	PHP_FE(drizzle_query_context, arginfo_drizzle_query_context)
	PHP_FE(drizzle_query_set_context, arginfo_drizzle_query_set_context)
	PHP_FE(drizzle_query_run, arginfo_drizzle_query_run)
	PHP_FE(drizzle_query_run_all, arginfo_drizzle_query_run_all)
	PHP_FE(drizzle_query_return_code, arginfo_drizzle_query_return_code)
	PHP_FE(drizzle_escape_string, arginfo_drizzle_escape_string)
	PHP_FE(drizzle_hex_string, arginfo_drizzle_hex_string)

	/* Functions from result.h */
	PHP_FE(drizzle_result_create, arginfo_drizzle_result_create)
	PHP_FE(drizzle_result_clone, arginfo_drizzle_result_clone)
	PHP_FE(drizzle_result_free, arginfo_drizzle_result_free)
	PHP_FE(drizzle_result_drizzle_con, arginfo_drizzle_result_drizzle_con)
	PHP_FE(drizzle_result_eof, arginfo_drizzle_result_eof)
	PHP_FE(drizzle_result_info, arginfo_drizzle_result_info)
	PHP_FE(drizzle_result_error, arginfo_drizzle_result_error)
	PHP_FE(drizzle_result_error_code, arginfo_drizzle_result_error_code)
	PHP_FE(drizzle_result_sqlstate, arginfo_drizzle_result_sqlstate)
	PHP_FE(drizzle_result_warning_count, arginfo_drizzle_result_warning_count)
	PHP_FE(drizzle_result_insert_id, arginfo_drizzle_result_insert_id)
	PHP_FE(drizzle_result_affected_rows, arginfo_drizzle_result_affected_rows)
	PHP_FE(drizzle_result_column_count, arginfo_drizzle_result_column_count)
	PHP_FE(drizzle_result_row_count, arginfo_drizzle_result_row_count)

	/* Functions from result_client.h */
	PHP_FE(drizzle_result_read, arginfo_drizzle_result_read)
	PHP_FE(drizzle_result_buffer, arginfo_drizzle_result_buffer)
	PHP_FE(drizzle_result_row_size, arginfo_drizzle_result_row_size)
	PHP_FE(drizzle_result_return_code, arginfo_drizzle_result_return_code)

	/* Functions from column.h */
	PHP_FE(drizzle_column_create, arginfo_drizzle_column_create)
	PHP_FE(drizzle_column_free, arginfo_drizzle_column_free)
	PHP_FE(drizzle_column_drizzle_result, arginfo_drizzle_column_drizzle_result)
	PHP_FE(drizzle_column_catalog, arginfo_drizzle_column_catalog)
	PHP_FE(drizzle_column_db, arginfo_drizzle_column_db)
	PHP_FE(drizzle_column_table, arginfo_drizzle_column_table)
	PHP_FE(drizzle_column_orig_table, arginfo_drizzle_column_orig_table)
	PHP_FE(drizzle_column_name, arginfo_drizzle_column_name)
	PHP_FE(drizzle_column_orig_name, arginfo_drizzle_column_orig_name)
	PHP_FE(drizzle_column_charset, arginfo_drizzle_column_charset)
	PHP_FE(drizzle_column_size, arginfo_drizzle_column_size)
	PHP_FE(drizzle_column_max_size, arginfo_drizzle_column_max_size)
	PHP_FE(drizzle_column_type, arginfo_drizzle_column_type)
	PHP_FE(drizzle_column_type_drizzle, arginfo_drizzle_column_type_drizzle)
	PHP_FE(drizzle_column_flags, arginfo_drizzle_column_flags)
	PHP_FE(drizzle_column_decimals, arginfo_drizzle_column_decimals)
	PHP_FE(drizzle_column_default_value, arginfo_drizzle_column_default_value)

	/* Functions from column_client.h */
	PHP_FE(drizzle_column_skip, arginfo_drizzle_column_skip)
	PHP_FE(drizzle_column_read, arginfo_drizzle_column_read)
	PHP_FE(drizzle_column_buffer, arginfo_drizzle_column_buffer)
	PHP_FE(drizzle_column_next, arginfo_drizzle_column_next)
	PHP_FE(drizzle_column_prev, arginfo_drizzle_column_prev)
	PHP_FE(drizzle_column_seek, arginfo_drizzle_column_seek)
	PHP_FE(drizzle_column_index, arginfo_drizzle_column_index)
	PHP_FE(drizzle_column_current, arginfo_drizzle_column_current)
	PHP_FE(drizzle_column_return_code, arginfo_drizzle_column_return_code)

	/* Functions from row_client.h */
	PHP_FE(drizzle_row_read, arginfo_drizzle_row_read)
	PHP_FE(drizzle_row_buffer, arginfo_drizzle_row_buffer)
	PHP_FE(drizzle_row_field_sizes, arginfo_drizzle_row_field_sizes)
	PHP_FE(drizzle_row_next, arginfo_drizzle_row_next)
	PHP_FE(drizzle_row_prev, arginfo_drizzle_row_prev)
	PHP_FE(drizzle_row_seek, arginfo_drizzle_row_seek)
	PHP_FE(drizzle_row_index, arginfo_drizzle_row_index)
	PHP_FE(drizzle_row_current, arginfo_drizzle_row_current)

	/* Functions from field_client.h */
	PHP_FE(drizzle_field_read, arginfo_drizzle_field_read)
	PHP_FE(drizzle_field_buffer, arginfo_drizzle_field_buffer)

	{NULL, NULL, NULL} /* Must be the last line. */
};

/* Methods for drizzle. */
zend_function_entry drizzle_methods[]= {
	PHP_ME(Drizzle, __construct, NULL, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	__PHP_ME_MAPPING(version, drizzle_version, arginfo_oo_drizzle_version, 0)
	__PHP_ME_MAPPING(clone, drizzle_clone, arginfo_oo_drizzle_clone, 0)
	__PHP_ME_MAPPING(error, drizzle_error, arginfo_oo_drizzle_error, 0)
	__PHP_ME_MAPPING(getErrno, drizzle_errno, arginfo_oo_drizzle_errno, 0)
	__PHP_ME_MAPPING(errorCode, drizzle_error_code, arginfo_oo_drizzle_error_code, 0)
	__PHP_ME_MAPPING(sqlstate, drizzle_sqlstate, arginfo_oo_drizzle_sqlstate, 0)
	__PHP_ME_MAPPING(options, drizzle_options, arginfo_oo_drizzle_options, 0)
	__PHP_ME_MAPPING(setOptions, drizzle_set_options, arginfo_oo_drizzle_set_options, 0)
	__PHP_ME_MAPPING(addOptions, drizzle_add_options, arginfo_oo_drizzle_add_options, 0)
	__PHP_ME_MAPPING(removeOptions, drizzle_remove_options, arginfo_oo_drizzle_remove_options, 0)
	__PHP_ME_MAPPING(wait, drizzle_con_wait, arginfo_oo_drizzle_con_wait, 0)
	__PHP_ME_MAPPING(ready, drizzle_con_ready, arginfo_oo_drizzle_con_ready, 0)
	__PHP_ME_MAPPING(conCreate, drizzle_con_create, arginfo_oo_drizzle_con_create, 0)
	__PHP_ME_MAPPING(addTcp, drizzle_con_add_tcp, arginfo_oo_drizzle_con_add_tcp, 0)
	__PHP_ME_MAPPING(addUds, drizzle_con_add_uds, arginfo_oo_drizzle_con_add_uds, 0)
	__PHP_ME_MAPPING(queryAdd, drizzle_query_add, arginfo_oo_drizzle_query_add, 0)
	__PHP_ME_MAPPING(queryCreate, drizzle_query_create, arginfo_oo_drizzle_query_create, 0)
	__PHP_ME_MAPPING(escapeString, drizzle_escape_string, arginfo_oo_drizzle_escape_string, 0)
	__PHP_ME_MAPPING(hex, drizzle_hex_string, arginfo_oo_drizzle_hex_string, 0)
	__PHP_ME_MAPPING(run, drizzle_query_run, arginfo_oo_drizzle_query_run, 0)
	__PHP_ME_MAPPING(runAll, drizzle_query_run_all, arginfo_oo_drizzle_query_run_all, 0)
	__PHP_ME_MAPPING(returnCode, drizzle_return_code, arginfo_oo_drizzle_return_code, 0)

	{NULL, NULL, NULL} /* Must be the last line. */
};

/* Methods for drizzle_con. */
zend_function_entry drizzle_con_methods[]= {
	PHP_ME(DrizzleCon, __construct, NULL, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	__PHP_ME_MAPPING(clone, drizzle_con_clone, arginfo_oo_drizzle_con_clone, 0)
	__PHP_ME_MAPPING(fd, drizzle_con_fd, arginfo_oo_drizzle_con_fd, 0)
	__PHP_ME_MAPPING(setFd, drizzle_con_set_fd, arginfo_oo_drizzle_con_set_fd, 0)
	__PHP_ME_MAPPING(close, drizzle_con_close, arginfo_oo_drizzle_con_close, 0)
	__PHP_ME_MAPPING(drizzle, drizzle_con_drizzle, arginfo_oo_drizzle_con_drizzle, 0)
	__PHP_ME_MAPPING(error, drizzle_con_error, arginfo_oo_drizzle_con_error, 0)
	__PHP_ME_MAPPING(getErrno, drizzle_con_errno, arginfo_oo_drizzle_con_errno, 0)
	__PHP_ME_MAPPING(errorCode, drizzle_con_error_code, arginfo_oo_drizzle_con_error_code, 0)
	__PHP_ME_MAPPING(sqlstate, drizzle_con_sqlstate, arginfo_oo_drizzle_con_sqlstate, 0)
	__PHP_ME_MAPPING(options, drizzle_con_options, arginfo_oo_drizzle_con_options, 0)
	__PHP_ME_MAPPING(setOptions, drizzle_con_set_options, arginfo_oo_drizzle_con_set_options, 0)
	__PHP_ME_MAPPING(addOptions, drizzle_con_add_options, arginfo_oo_drizzle_con_add_options, 0)
	__PHP_ME_MAPPING(removeOptions, drizzle_con_remove_options, arginfo_oo_drizzle_con_remove_options, 0)
	__PHP_ME_MAPPING(host, drizzle_con_host, arginfo_oo_drizzle_con_host, 0)
	__PHP_ME_MAPPING(port, drizzle_con_port, arginfo_oo_drizzle_con_port, 0)
	__PHP_ME_MAPPING(setTcp, drizzle_con_set_tcp, arginfo_oo_drizzle_con_set_tcp, 0)
	__PHP_ME_MAPPING(uds, drizzle_con_uds, arginfo_oo_drizzle_con_uds, 0)
	__PHP_ME_MAPPING(setUds, drizzle_con_set_uds, arginfo_oo_drizzle_con_set_uds, 0)
	__PHP_ME_MAPPING(user, drizzle_con_user, arginfo_oo_drizzle_con_user, 0)
	__PHP_ME_MAPPING(password, drizzle_con_password, arginfo_oo_drizzle_con_password, 0)
	__PHP_ME_MAPPING(setAuth, drizzle_con_set_auth, arginfo_oo_drizzle_con_set_auth, 0)
	__PHP_ME_MAPPING(db, drizzle_con_db, arginfo_oo_drizzle_con_db, 0)
	__PHP_ME_MAPPING(setDb, drizzle_con_set_db, arginfo_oo_drizzle_con_set_db, 0)
	__PHP_ME_MAPPING(context, drizzle_con_context, arginfo_oo_drizzle_con_context, 0)
	__PHP_ME_MAPPING(setContext, drizzle_con_set_context, arginfo_oo_drizzle_con_set_context, 0)
	__PHP_ME_MAPPING(protocolVersion, drizzle_con_protocol_version, arginfo_oo_drizzle_con_protocol_version, 0)
	__PHP_ME_MAPPING(serverVersion, drizzle_con_server_version, arginfo_oo_drizzle_con_server_version, 0)
	__PHP_ME_MAPPING(serverVersionNumber, drizzle_con_server_version_number, 
	               arginfo_oo_drizzle_con_server_version_number, 0)
	__PHP_ME_MAPPING(threadId, drizzle_con_thread_id, arginfo_oo_drizzle_con_thread_id, 0)
	__PHP_ME_MAPPING(scramble, drizzle_con_scramble, arginfo_oo_drizzle_con_scramble, 0)
	__PHP_ME_MAPPING(capabilities, drizzle_con_capabilities, arginfo_oo_drizzle_con_capabilities, 0)
	__PHP_ME_MAPPING(charset, drizzle_con_charset, arginfo_oo_drizzle_con_charset, 0)
	__PHP_ME_MAPPING(status, drizzle_con_status, arginfo_oo_drizzle_con_status, 0)
	__PHP_ME_MAPPING(maxPacketSize, drizzle_con_max_packet_size, arginfo_oo_drizzle_con_max_packet_size, 0)
	__PHP_ME_MAPPING(connect, drizzle_con_connect, arginfo_oo_drizzle_con_connect, 0)
	__PHP_ME_MAPPING(serverHandshakeRead, drizzle_handshake_server_read, 
	               arginfo_oo_drizzle_handshake_server_read, 0)
	__PHP_ME_MAPPING(clientHandshakeWrite, drizzle_handshake_client_write, 
	               arginfo_oo_drizzle_handshake_client_write, 0)
	__PHP_ME_MAPPING(quit, drizzle_quit, arginfo_oo_drizzle_quit, 0)
	__PHP_ME_MAPPING(selectDb, drizzle_select_db, arginfo_oo_drizzle_select_db, 0)
	__PHP_ME_MAPPING(shutdown, drizzle_shutdown, arginfo_oo_drizzle_shutdown, 0)
	__PHP_ME_MAPPING(ping, drizzle_ping, arginfo_oo_drizzle_ping, 0)
	__PHP_ME_MAPPING(commandWrite, drizzle_con_command_write, arginfo_oo_drizzle_con_command_write, 0)
	__PHP_ME_MAPPING(query, drizzle_query, arginfo_oo_drizzle_query, 0)
	__PHP_ME_MAPPING(read, drizzle_result_read, arginfo_oo_drizzle_result_read, 0)
	__PHP_ME_MAPPING(returnCode, drizzle_con_return_code, arginfo_oo_drizzle_con_return_code, 0)
	__PHP_ME_MAPPING(resultCreate, drizzle_result_create, arginfo_oo_drizzle_result_create, 0)

	{NULL, NULL, NULL} /* Must be the last line. */
};

/* Methods for drizzle_column. */
zend_function_entry drizzle_query_methods[]= { 
	PHP_ME(DrizzleQuery, __construct, NULL, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	__PHP_ME_MAPPING(con, drizzle_query_con, arginfo_oo_drizzle_query_con, 0)
	__PHP_ME_MAPPING(setCon, drizzle_query_set_con, arginfo_oo_drizzle_query_set_con, 0)
	__PHP_ME_MAPPING(result, drizzle_query_result, arginfo_oo_drizzle_query_result, 0)
	__PHP_ME_MAPPING(string, drizzle_query_string, arginfo_oo_drizzle_query_string, 0)
	__PHP_ME_MAPPING(setString, drizzle_query_set_string, arginfo_oo_drizzle_query_set_string, 0)
	__PHP_ME_MAPPING(options, drizzle_query_options, arginfo_oo_drizzle_query_options, 0)
	__PHP_ME_MAPPING(setOptions, drizzle_query_set_options, arginfo_oo_drizzle_query_set_options, 0)
	__PHP_ME_MAPPING(addOptions, drizzle_query_add_options, arginfo_oo_drizzle_query_add_options, 0)
	__PHP_ME_MAPPING(removeOptions, drizzle_query_remove_options, arginfo_oo_drizzle_query_remove_options, 0)
	__PHP_ME_MAPPING(context, drizzle_query_context, arginfo_oo_drizzle_query_context, 0)
	__PHP_ME_MAPPING(setContext, drizzle_query_set_context, arginfo_oo_drizzle_query_set_context, 0)
	__PHP_ME_MAPPING(returnCode, drizzle_query_return_code, arginfo_oo_drizzle_query_return_code, 0)

	{NULL, NULL, NULL} /* Must be the last line. */
};

/* Methods for drizzle_result. */
zend_function_entry drizzle_result_methods[]= {
	PHP_ME(DrizzleResult, __construct, NULL, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	__PHP_ME_MAPPING(clone, drizzle_result_clone, arginfo_oo_drizzle_result_clone, 0)
	__PHP_ME_MAPPING(con, drizzle_result_drizzle_con, arginfo_oo_drizzle_result_drizzle_con, 0)
	__PHP_ME_MAPPING(eof, drizzle_result_eof, arginfo_oo_drizzle_result_eof, 0)
	__PHP_ME_MAPPING(info, drizzle_result_info, arginfo_oo_drizzle_result_info, 0)
	__PHP_ME_MAPPING(error, drizzle_result_error, arginfo_oo_drizzle_result_error, 0)
	__PHP_ME_MAPPING(errorCode, drizzle_result_error_code, arginfo_oo_drizzle_result_error_code, 0)
	__PHP_ME_MAPPING(sqlstate, drizzle_result_sqlstate, arginfo_oo_drizzle_result_sqlstate, 0)
	__PHP_ME_MAPPING(warningCount, drizzle_result_warning_count, arginfo_oo_drizzle_result_warning_count, 0)
	__PHP_ME_MAPPING(insertId, drizzle_result_insert_id, arginfo_oo_drizzle_result_insert_id, 0)
	__PHP_ME_MAPPING(affectedRows, drizzle_result_affected_rows, arginfo_oo_drizzle_result_affected_rows, 0)
	__PHP_ME_MAPPING(columnCount, drizzle_result_column_count, arginfo_oo_drizzle_result_column_count, 0)
	__PHP_ME_MAPPING(rowCount, drizzle_result_row_count, arginfo_oo_drizzle_result_row_count, 0)
	__PHP_ME_MAPPING(buffer, drizzle_result_buffer, arginfo_oo_drizzle_result_buffer, 0)
	__PHP_ME_MAPPING(rowSize, drizzle_result_row_size, arginfo_oo_drizzle_result_row_size, 0)
	__PHP_ME_MAPPING(columnCreate, drizzle_column_create, arginfo_oo_drizzle_column_create, 0)
	__PHP_ME_MAPPING(columnSkip, drizzle_column_skip, arginfo_oo_drizzle_column_skip, 0)
	__PHP_ME_MAPPING(columnRead, drizzle_column_read, arginfo_oo_drizzle_column_read, 0)
	__PHP_ME_MAPPING(columnBuffer, drizzle_column_buffer, arginfo_oo_drizzle_column_buffer, 0)
	__PHP_ME_MAPPING(columnNext, drizzle_column_next, arginfo_oo_drizzle_column_next, 0)
	__PHP_ME_MAPPING(columnPrev, drizzle_column_prev, arginfo_oo_drizzle_column_prev, 0)
	__PHP_ME_MAPPING(columnSeek, drizzle_column_seek, arginfo_oo_drizzle_column_seek, 0)
	__PHP_ME_MAPPING(columnIndex, drizzle_column_index, arginfo_oo_drizzle_column_index, 0)
	__PHP_ME_MAPPING(columnCurrent, drizzle_column_current, arginfo_oo_drizzle_column_current, 0)
	__PHP_ME_MAPPING(rowRead, drizzle_row_read, arginfo_oo_drizzle_row_read, 0)
	__PHP_ME_MAPPING(rowBuffer, drizzle_row_buffer, arginfo_oo_drizzle_row_buffer, 0)
	__PHP_ME_MAPPING(rowField_sizes, drizzle_row_field_sizes, arginfo_oo_drizzle_row_field_sizes, 0)
	__PHP_ME_MAPPING(rowNext, drizzle_row_next, arginfo_oo_drizzle_row_next, 0)
	__PHP_ME_MAPPING(rowPrev, drizzle_row_prev, arginfo_oo_drizzle_row_prev, 0)
	__PHP_ME_MAPPING(rowSeek, drizzle_row_seek, arginfo_oo_drizzle_row_seek, 0)
	__PHP_ME_MAPPING(rowIndex, drizzle_row_index, arginfo_oo_drizzle_row_index, 0)
	__PHP_ME_MAPPING(rowCurrent, drizzle_row_current, arginfo_oo_drizzle_row_current, 0)
	__PHP_ME_MAPPING(fieldRead, drizzle_field_read, arginfo_oo_drizzle_field_read, 0)
	__PHP_ME_MAPPING(fieldBuffer, drizzle_field_buffer, arginfo_oo_drizzle_field_buffer, 0)
	__PHP_ME_MAPPING(returnCode, drizzle_result_return_code, arginfo_oo_drizzle_result_return_code, 0)

	{NULL, NULL, NULL} /* Must be the last line. */
};

/* Methods for drizzle_column. */
zend_function_entry drizzle_column_methods[]= { 
	PHP_ME(DrizzleColumn, __construct, NULL, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	__PHP_ME_MAPPING(result, drizzle_column_drizzle_result, arginfo_oo_drizzle_column_drizzle_result, 0)
	__PHP_ME_MAPPING(catalog, drizzle_column_catalog, arginfo_oo_drizzle_column_catalog, 0)
	__PHP_ME_MAPPING(db, drizzle_column_db, arginfo_oo_drizzle_column_db, 0)
	__PHP_ME_MAPPING(table, drizzle_column_table, arginfo_oo_drizzle_column_table, 0)
	__PHP_ME_MAPPING(origTable, drizzle_column_orig_table, arginfo_oo_drizzle_column_orig_table, 0)
	__PHP_ME_MAPPING(name, drizzle_column_name, arginfo_oo_drizzle_column_name, 0)
	__PHP_ME_MAPPING(origName, drizzle_column_orig_name, arginfo_oo_drizzle_column_orig_name, 0)
	__PHP_ME_MAPPING(charset, drizzle_column_charset, arginfo_oo_drizzle_column_charset, 0)
	__PHP_ME_MAPPING(size, drizzle_column_size, arginfo_oo_drizzle_column_size, 0)
	__PHP_ME_MAPPING(maxSize, drizzle_column_max_size, arginfo_oo_drizzle_column_max_size, 0)
	__PHP_ME_MAPPING(type, drizzle_column_type, arginfo_oo_drizzle_column_type, 0)
	__PHP_ME_MAPPING(typeDrizzle, drizzle_column_type_drizzle, arginfo_oo_drizzle_column_type_drizzle, 0)
	__PHP_ME_MAPPING(flags, drizzle_column_flags, arginfo_oo_drizzle_column_flags, 0)
	__PHP_ME_MAPPING(decimals, drizzle_column_decimals, arginfo_oo_drizzle_column_decimals, 0)
	__PHP_ME_MAPPING(defaultValue, drizzle_column_default_value, arginfo_oo_drizzle_column_default_value, 0)
	__PHP_ME_MAPPING(returnCode, drizzle_column_return_code, arginfo_oo_drizzle_column_return_code, 0)

	{NULL, NULL, NULL} /* Must be the last line. */
};

zend_function_entry drizzle_exception_methods[] = {
	{NULL, NULL, NULL}
};

PHP_MINIT_FUNCTION(drizzle) {
	zend_class_entry ce;

	INIT_CLASS_ENTRY(ce, "Drizzle", drizzle_methods)
	ce.create_object= drizzle_obj_new;
	drizzle_ce= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	memcpy(&drizzle_obj_handlers, zend_get_std_object_handlers(),
	       sizeof(zend_object_handlers));
	drizzle_obj_handlers.clone_obj= NULL;

	INIT_CLASS_ENTRY(ce, "DrizzleCon", drizzle_con_methods)
	ce.create_object= drizzle_con_obj_new;
	drizzle_con_ce= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	memcpy(&drizzle_con_obj_handlers, zend_get_std_object_handlers(),
	       sizeof(zend_object_handlers));
	drizzle_con_obj_handlers.clone_obj= NULL;

	INIT_CLASS_ENTRY(ce, "DrizzleQuery", drizzle_query_methods)
	ce.create_object= drizzle_query_obj_new;
	drizzle_query_ce= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	memcpy(&drizzle_query_obj_handlers, zend_get_std_object_handlers(),
	       sizeof(zend_object_handlers));
	drizzle_query_obj_handlers.clone_obj= NULL;

	INIT_CLASS_ENTRY(ce, "DrizzleResult", drizzle_result_methods)
	ce.create_object= drizzle_result_obj_new;
	drizzle_result_ce= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	memcpy(&drizzle_result_obj_handlers, zend_get_std_object_handlers(),
	       sizeof(zend_object_handlers));
	drizzle_result_obj_handlers.clone_obj= NULL;

	INIT_CLASS_ENTRY(ce, "DrizzleColumn", drizzle_column_methods)
	ce.create_object= drizzle_column_obj_new;
	drizzle_column_ce= zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
	memcpy(&drizzle_column_obj_handlers, zend_get_std_object_handlers(),
	       sizeof(zend_object_handlers));
	drizzle_column_obj_handlers.clone_obj= NULL;

	/* XXX exception class */
	INIT_CLASS_ENTRY(ce, "DrizzleException", drizzle_exception_methods)
	drizzle_exception_ce = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);
	drizzle_exception_ce->ce_flags |= ZEND_ACC_FINAL;
	zend_declare_property_long(drizzle_exception_ce, "code", sizeof("code")-1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);


  /* These are automatically generated from constants.h using const_gen.sh. Do
     not remove the CONST_GEN_* comments, this is how the script locates the
     correct location to replace. */

  /* CONST_GEN_START */
  REGISTER_LONG_CONSTANT("DRIZZLE_ALLOCATED",
                         DRIZZLE_ALLOCATED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_ASSERT_DANGLING",
                         DRIZZLE_ASSERT_DANGLING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_BUFFER_COPY_THRESHOLD",
                         DRIZZLE_BUFFER_COPY_THRESHOLD,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_CLIENT",
                         DRIZZLE_CAPABILITIES_CLIENT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_COMPRESS",
                         DRIZZLE_CAPABILITIES_COMPRESS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_CONNECT_WITH_DB",
                         DRIZZLE_CAPABILITIES_CONNECT_WITH_DB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_FOUND_ROWS",
                         DRIZZLE_CAPABILITIES_FOUND_ROWS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_IGNORE_SIGPIPE",
                         DRIZZLE_CAPABILITIES_IGNORE_SIGPIPE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_IGNORE_SPACE",
                         DRIZZLE_CAPABILITIES_IGNORE_SPACE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_INTERACTIVE",
                         DRIZZLE_CAPABILITIES_INTERACTIVE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_LOCAL_FILES",
                         DRIZZLE_CAPABILITIES_LOCAL_FILES,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_LONG_FLAG",
                         DRIZZLE_CAPABILITIES_LONG_FLAG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_LONG_PASSWORD",
                         DRIZZLE_CAPABILITIES_LONG_PASSWORD,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_MULTI_RESULTS",
                         DRIZZLE_CAPABILITIES_MULTI_RESULTS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_MULTI_STATEMENTS",
                         DRIZZLE_CAPABILITIES_MULTI_STATEMENTS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_NONE",
                         DRIZZLE_CAPABILITIES_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_NO_SCHEMA",
                         DRIZZLE_CAPABILITIES_NO_SCHEMA,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_ODBC",
                         DRIZZLE_CAPABILITIES_ODBC,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_PROTOCOL_41",
                         DRIZZLE_CAPABILITIES_PROTOCOL_41,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_REMEMBER_OPTIONS",
                         DRIZZLE_CAPABILITIES_REMEMBER_OPTIONS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_RESERVED",
                         DRIZZLE_CAPABILITIES_RESERVED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_SECURE_CONNECTION",
                         DRIZZLE_CAPABILITIES_SECURE_CONNECTION,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_SSL",
                         DRIZZLE_CAPABILITIES_SSL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_SSL_VERIFY_SERVER_CERT",
                         DRIZZLE_CAPABILITIES_SSL_VERIFY_SERVER_CERT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CAPABILITIES_TRANSACTIONS",
                         DRIZZLE_CAPABILITIES_TRANSACTIONS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_ALLOCATED",
                         DRIZZLE_COLUMN_ALLOCATED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_AUTO_INCREMENT",
                         DRIZZLE_COLUMN_FLAGS_AUTO_INCREMENT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_BINARY",
                         DRIZZLE_COLUMN_FLAGS_BINARY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_BINCMP",
                         DRIZZLE_COLUMN_FLAGS_BINCMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_BLOB",
                         DRIZZLE_COLUMN_FLAGS_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_ENUM",
                         DRIZZLE_COLUMN_FLAGS_ENUM,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_GET_FIXED_FIELDS",
                         DRIZZLE_COLUMN_FLAGS_GET_FIXED_FIELDS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_GROUP",
                         DRIZZLE_COLUMN_FLAGS_GROUP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_IN_ADD_INDEX",
                         DRIZZLE_COLUMN_FLAGS_IN_ADD_INDEX,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_IN_PART_FUNC",
                         DRIZZLE_COLUMN_FLAGS_IN_PART_FUNC,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_MULTIPLE_KEY",
                         DRIZZLE_COLUMN_FLAGS_MULTIPLE_KEY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_NO_DEFAULT_VALUE",
                         DRIZZLE_COLUMN_FLAGS_NO_DEFAULT_VALUE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_NONE",
                         DRIZZLE_COLUMN_FLAGS_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_NOT_NULL",
                         DRIZZLE_COLUMN_FLAGS_NOT_NULL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_NUM",
                         DRIZZLE_COLUMN_FLAGS_NUM,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_ON_UPDATE_NOW",
                         DRIZZLE_COLUMN_FLAGS_ON_UPDATE_NOW,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_PART_KEY",
                         DRIZZLE_COLUMN_FLAGS_PART_KEY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_PRI_KEY",
                         DRIZZLE_COLUMN_FLAGS_PRI_KEY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_RENAMED",
                         DRIZZLE_COLUMN_FLAGS_RENAMED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_SET",
                         DRIZZLE_COLUMN_FLAGS_SET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_TIMESTAMP",
                         DRIZZLE_COLUMN_FLAGS_TIMESTAMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_UNIQUE",
                         DRIZZLE_COLUMN_FLAGS_UNIQUE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_UNIQUE_KEY",
                         DRIZZLE_COLUMN_FLAGS_UNIQUE_KEY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_UNSIGNED",
                         DRIZZLE_COLUMN_FLAGS_UNSIGNED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_FLAGS_ZEROFILL",
                         DRIZZLE_COLUMN_FLAGS_ZEROFILL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_BIT",
                         DRIZZLE_COLUMN_TYPE_BIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_BLOB",
                         DRIZZLE_COLUMN_TYPE_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DATE",
                         DRIZZLE_COLUMN_TYPE_DATE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DATETIME",
                         DRIZZLE_COLUMN_TYPE_DATETIME,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DECIMAL",
                         DRIZZLE_COLUMN_TYPE_DECIMAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DOUBLE",
                         DRIZZLE_COLUMN_TYPE_DOUBLE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_BLOB",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_DATE",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_DATE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_DATETIME",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_DATETIME,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_DOUBLE",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_DOUBLE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_ENUM",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_ENUM,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_LONG",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_LONG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_LONGLONG",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_LONGLONG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_MAX,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_NEWDECIMAL",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_NEWDECIMAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_NULL",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_NULL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_TIMESTAMP",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_TIMESTAMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_TINY",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_TINY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_DRIZZLE_VARCHAR",
                         DRIZZLE_COLUMN_TYPE_DRIZZLE_VARCHAR,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_ENUM",
                         DRIZZLE_COLUMN_TYPE_ENUM,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_FLOAT",
                         DRIZZLE_COLUMN_TYPE_FLOAT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_GEOMETRY",
                         DRIZZLE_COLUMN_TYPE_GEOMETRY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_INT24",
                         DRIZZLE_COLUMN_TYPE_INT24,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_LONG",
                         DRIZZLE_COLUMN_TYPE_LONG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_LONG_BLOB",
                         DRIZZLE_COLUMN_TYPE_LONG_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_LONGLONG",
                         DRIZZLE_COLUMN_TYPE_LONGLONG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_MEDIUM_BLOB",
                         DRIZZLE_COLUMN_TYPE_MEDIUM_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_NEWDATE",
                         DRIZZLE_COLUMN_TYPE_NEWDATE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_NEWDECIMAL",
                         DRIZZLE_COLUMN_TYPE_NEWDECIMAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_NULL",
                         DRIZZLE_COLUMN_TYPE_NULL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_SET",
                         DRIZZLE_COLUMN_TYPE_SET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_SHORT",
                         DRIZZLE_COLUMN_TYPE_SHORT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_STRING",
                         DRIZZLE_COLUMN_TYPE_STRING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_TIME",
                         DRIZZLE_COLUMN_TYPE_TIME,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_TIMESTAMP",
                         DRIZZLE_COLUMN_TYPE_TIMESTAMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_TINY",
                         DRIZZLE_COLUMN_TYPE_TINY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_TINY_BLOB",
                         DRIZZLE_COLUMN_TYPE_TINY_BLOB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_VARCHAR",
                         DRIZZLE_COLUMN_TYPE_VARCHAR,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_VAR_STRING",
                         DRIZZLE_COLUMN_TYPE_VAR_STRING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_VIRTUAL",
                         DRIZZLE_COLUMN_TYPE_VIRTUAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COLUMN_TYPE_YEAR",
                         DRIZZLE_COLUMN_TYPE_YEAR,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_BINLOG_DUMP",
                         DRIZZLE_COMMAND_BINLOG_DUMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_CHANGE_USER",
                         DRIZZLE_COMMAND_CHANGE_USER,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_CONNECT",
                         DRIZZLE_COMMAND_CONNECT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_CONNECT_OUT",
                         DRIZZLE_COMMAND_CONNECT_OUT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_CREATE_DB",
                         DRIZZLE_COMMAND_CREATE_DB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DAEMON",
                         DRIZZLE_COMMAND_DAEMON,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DEBUG",
                         DRIZZLE_COMMAND_DEBUG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DELAYED_INSERT",
                         DRIZZLE_COMMAND_DELAYED_INSERT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_CONNECT",
                         DRIZZLE_COMMAND_DRIZZLE_CONNECT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_END",
                         DRIZZLE_COMMAND_DRIZZLE_END,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_INIT_DB",
                         DRIZZLE_COMMAND_DRIZZLE_INIT_DB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_PING",
                         DRIZZLE_COMMAND_DRIZZLE_PING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_QUERY",
                         DRIZZLE_COMMAND_DRIZZLE_QUERY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_QUIT",
                         DRIZZLE_COMMAND_DRIZZLE_QUIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_SHUTDOWN",
                         DRIZZLE_COMMAND_DRIZZLE_SHUTDOWN,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DRIZZLE_SLEEP",
                         DRIZZLE_COMMAND_DRIZZLE_SLEEP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_DROP_DB",
                         DRIZZLE_COMMAND_DROP_DB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_END",
                         DRIZZLE_COMMAND_END,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_FIELD_LIST",
                         DRIZZLE_COMMAND_FIELD_LIST,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_INIT_DB",
                         DRIZZLE_COMMAND_INIT_DB,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_PING",
                         DRIZZLE_COMMAND_PING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_PROCESS_INFO",
                         DRIZZLE_COMMAND_PROCESS_INFO,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_PROCESS_KILL",
                         DRIZZLE_COMMAND_PROCESS_KILL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_QUERY",
                         DRIZZLE_COMMAND_QUERY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_QUIT",
                         DRIZZLE_COMMAND_QUIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_REFRESH",
                         DRIZZLE_COMMAND_REFRESH,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_REGISTER_SLAVE",
                         DRIZZLE_COMMAND_REGISTER_SLAVE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_SET_OPTION",
                         DRIZZLE_COMMAND_SET_OPTION,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_SHUTDOWN",
                         DRIZZLE_COMMAND_SHUTDOWN,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_SLEEP",
                         DRIZZLE_COMMAND_SLEEP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STATISTICS",
                         DRIZZLE_COMMAND_STATISTICS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_CLOSE",
                         DRIZZLE_COMMAND_STMT_CLOSE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_EXECUTE",
                         DRIZZLE_COMMAND_STMT_EXECUTE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_FETCH",
                         DRIZZLE_COMMAND_STMT_FETCH,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_PREPARE",
                         DRIZZLE_COMMAND_STMT_PREPARE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_RESET",
                         DRIZZLE_COMMAND_STMT_RESET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_STMT_SEND_LONG_DATA",
                         DRIZZLE_COMMAND_STMT_SEND_LONG_DATA,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_TABLE_DUMP",
                         DRIZZLE_COMMAND_TABLE_DUMP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_COMMAND_TIME",
                         DRIZZLE_COMMAND_TIME,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_ALLOCATED",
                         DRIZZLE_CON_ALLOCATED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_EXPERIMENTAL",
                         DRIZZLE_CON_EXPERIMENTAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_IO_READY",
                         DRIZZLE_CON_IO_READY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_LISTEN",
                         DRIZZLE_CON_LISTEN,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_MYSQL",
                         DRIZZLE_CON_MYSQL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_NONE",
                         DRIZZLE_CON_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_NO_RESULT_READ",
                         DRIZZLE_CON_NO_RESULT_READ,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_RAW_PACKET",
                         DRIZZLE_CON_RAW_PACKET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_RAW_SCRAMBLE",
                         DRIZZLE_CON_RAW_SCRAMBLE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_READY",
                         DRIZZLE_CON_READY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_SOCKET_TCP",
                         DRIZZLE_CON_SOCKET_TCP,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_SOCKET_UDS",
                         DRIZZLE_CON_SOCKET_UDS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_AUTOCOMMIT",
                         DRIZZLE_CON_STATUS_AUTOCOMMIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_CURSOR_EXISTS",
                         DRIZZLE_CON_STATUS_CURSOR_EXISTS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_DB_DROPPED",
                         DRIZZLE_CON_STATUS_DB_DROPPED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_IN_TRANS",
                         DRIZZLE_CON_STATUS_IN_TRANS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_LAST_ROW_SENT",
                         DRIZZLE_CON_STATUS_LAST_ROW_SENT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS",
                         DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_NO_BACKSLASH_ESCAPES",
                         DRIZZLE_CON_STATUS_NO_BACKSLASH_ESCAPES,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_NONE",
                         DRIZZLE_CON_STATUS_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_QUERY_NO_GOOD_INDEX_USED",
                         DRIZZLE_CON_STATUS_QUERY_NO_GOOD_INDEX_USED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_QUERY_NO_INDEX_USED",
                         DRIZZLE_CON_STATUS_QUERY_NO_INDEX_USED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_CON_STATUS_QUERY_WAS_SLOW",
                         DRIZZLE_CON_STATUS_QUERY_WAS_SLOW,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_BACKLOG",
                         DRIZZLE_DEFAULT_BACKLOG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_SOCKET_RECV_SIZE",
                         DRIZZLE_DEFAULT_SOCKET_RECV_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_SOCKET_SEND_SIZE",
                         DRIZZLE_DEFAULT_SOCKET_SEND_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_SOCKET_TIMEOUT",
                         DRIZZLE_DEFAULT_SOCKET_TIMEOUT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_STRING_CONSTANT("DRIZZLE_DEFAULT_TCP_HOST",
                         DRIZZLE_DEFAULT_TCP_HOST,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_TCP_PORT",
                         DRIZZLE_DEFAULT_TCP_PORT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_DEFAULT_TCP_PORT_MYSQL",
                         DRIZZLE_DEFAULT_TCP_PORT_MYSQL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_STRING_CONSTANT("DRIZZLE_DEFAULT_UDS",
                         DRIZZLE_DEFAULT_UDS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_STRING_CONSTANT("DRIZZLE_DEFAULT_UDS_MYSQL",
                         DRIZZLE_DEFAULT_UDS_MYSQL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_FREE_OBJECTS",
                         DRIZZLE_FREE_OBJECTS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_BUFFER_SIZE",
                         DRIZZLE_MAX_BUFFER_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_CATALOG_SIZE",
                         DRIZZLE_MAX_CATALOG_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_COLUMN_NAME_SIZE",
                         DRIZZLE_MAX_COLUMN_NAME_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_DB_SIZE",
                         DRIZZLE_MAX_DB_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_DEFAULT_VALUE_SIZE",
                         DRIZZLE_MAX_DEFAULT_VALUE_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_ERROR_SIZE",
                         DRIZZLE_MAX_ERROR_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_INFO_SIZE",
                         DRIZZLE_MAX_INFO_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_PACKET_SIZE",
                         DRIZZLE_MAX_PACKET_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_PASSWORD_SIZE",
                         DRIZZLE_MAX_PASSWORD_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_SCRAMBLE_SIZE",
                         DRIZZLE_MAX_SCRAMBLE_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_SERVER_VERSION_SIZE",
                         DRIZZLE_MAX_SERVER_VERSION_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_SQLSTATE_SIZE",
                         DRIZZLE_MAX_SQLSTATE_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_TABLE_SIZE",
                         DRIZZLE_MAX_TABLE_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_MAX_USER_SIZE",
                         DRIZZLE_MAX_USER_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_NON_BLOCKING",
                         DRIZZLE_NON_BLOCKING,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_NONE",
                         DRIZZLE_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_QUERY_ALLOCATED",
                         DRIZZLE_QUERY_ALLOCATED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_QUERY_STATE_DONE",
                         DRIZZLE_QUERY_STATE_DONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_QUERY_STATE_INIT",
                         DRIZZLE_QUERY_STATE_INIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_QUERY_STATE_QUERY",
                         DRIZZLE_QUERY_STATE_QUERY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_QUERY_STATE_RESULT",
                         DRIZZLE_QUERY_STATE_RESULT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_ALLOCATED",
                         DRIZZLE_RESULT_ALLOCATED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_BUFFER_COLUMN",
                         DRIZZLE_RESULT_BUFFER_COLUMN,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_BUFFER_ROW",
                         DRIZZLE_RESULT_BUFFER_ROW,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_EOF_PACKET",
                         DRIZZLE_RESULT_EOF_PACKET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_NONE",
                         DRIZZLE_RESULT_NONE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_ROW_BREAK",
                         DRIZZLE_RESULT_ROW_BREAK,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RESULT_SKIP_COLUMN",
                         DRIZZLE_RESULT_SKIP_COLUMN,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_AUTH_FAILED",
                         DRIZZLE_RETURN_AUTH_FAILED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET",
                         DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_BAD_PACKET",
                         DRIZZLE_RETURN_BAD_PACKET,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_BAD_PACKET_NUMBER",
                         DRIZZLE_RETURN_BAD_PACKET_NUMBER,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_COULD_NOT_CONNECT",
                         DRIZZLE_RETURN_COULD_NOT_CONNECT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_ERRNO",
                         DRIZZLE_RETURN_ERRNO,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_ERROR_CODE",
                         DRIZZLE_RETURN_ERROR_CODE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_GETADDRINFO",
                         DRIZZLE_RETURN_GETADDRINFO,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_HANDSHAKE_FAILED",
                         DRIZZLE_RETURN_HANDSHAKE_FAILED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_INTERNAL_ERROR",
                         DRIZZLE_RETURN_INTERNAL_ERROR,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_IO_WAIT",
                         DRIZZLE_RETURN_IO_WAIT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_LOST_CONNECTION",
                         DRIZZLE_RETURN_LOST_CONNECTION,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_MAX",
                         DRIZZLE_RETURN_MAX,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_MEMORY",
                         DRIZZLE_RETURN_MEMORY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_NO_ACTIVE_CONNECTIONS",
                         DRIZZLE_RETURN_NO_ACTIVE_CONNECTIONS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_NO_SCRAMBLE",
                         DRIZZLE_RETURN_NO_SCRAMBLE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_NOT_READY",
                         DRIZZLE_RETURN_NOT_READY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_NULL_SIZE",
                         DRIZZLE_RETURN_NULL_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_OK",
                         DRIZZLE_RETURN_OK,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_PAUSE",
                         DRIZZLE_RETURN_PAUSE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED",
                         DRIZZLE_RETURN_PROTOCOL_NOT_SUPPORTED,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_ROW_BREAK",
                         DRIZZLE_RETURN_ROW_BREAK,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_ROW_END",
                         DRIZZLE_RETURN_ROW_END,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_TIMEOUT",
                         DRIZZLE_RETURN_TIMEOUT,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_TOO_MANY_COLUMNS",
                         DRIZZLE_RETURN_TOO_MANY_COLUMNS,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_RETURN_UNEXPECTED_DATA",
                         DRIZZLE_RETURN_UNEXPECTED_DATA,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_ROW_GROW_SIZE",
                         DRIZZLE_ROW_GROW_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_STATE_STACK_SIZE",
                         DRIZZLE_STATE_STACK_SIZE,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_CRAZY",
                         DRIZZLE_VERBOSE_CRAZY,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_DEBUG",
                         DRIZZLE_VERBOSE_DEBUG,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_ERROR",
                         DRIZZLE_VERBOSE_ERROR,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_FATAL",
                         DRIZZLE_VERBOSE_FATAL,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_INFO",
                         DRIZZLE_VERBOSE_INFO,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_MAX",
                         DRIZZLE_VERBOSE_MAX,
                         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DRIZZLE_VERBOSE_NEVER",
                         DRIZZLE_VERBOSE_NEVER,
                         CONST_CS | CONST_PERSISTENT);
  /* CONST_GEN_STOP */

	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(drizzle) {
	return SUCCESS;
}

PHP_MINFO_FUNCTION(drizzle) {
	char port_str[6];

	php_info_print_table_start();
	php_info_print_table_header(2, "drizzle support", "enabled");
	php_info_print_table_row(2, "extension Version", PHP_DRIZZLE_VERSION);
	php_info_print_table_row(2, "libdrizzle version", drizzle_version());
	php_info_print_table_row(2, "default TCP Host", DRIZZLE_DEFAULT_TCP_HOST);
	snprintf(port_str, 6, "%u", DRIZZLE_DEFAULT_TCP_PORT);
	php_info_print_table_row(2, "default TCP Port", port_str);
	snprintf(port_str, 6, "%u", DRIZZLE_DEFAULT_TCP_PORT_MYSQL);
	php_info_print_table_row(2, "default MySQL TCP Port", port_str);
	php_info_print_table_row(2, "default Unix Domain Socket",
	                         DRIZZLE_DEFAULT_UDS);
	php_info_print_table_row(2, "default MySQL Unix Domain Socket",
	                         DRIZZLE_DEFAULT_UDS_MYSQL);
	php_info_print_table_end();
}

/* Module config struct. */
zend_module_entry drizzle_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"drizzle",
	drizzle_functions,
	PHP_MINIT(drizzle),
	PHP_MSHUTDOWN(drizzle),
	NULL,
	NULL,
	PHP_MINFO(drizzle),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1",
#endif
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_DRIZZLE
ZEND_GET_MODULE(drizzle)
#endif
